/*
*  E1.1.0 RAMS Increment 1 Patch Code
*  January 2015
*  Department of Veterans Affairs/Booz Allen Hamilton
*/

package gov.va.rams.docx4j;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;
import java.io.InputStream;
import java.util.ArrayList;

import org.docx4j.TraversalUtil;
import org.docx4j.XmlUtils;
import org.docx4j.finders.RangeFinder;
import org.docx4j.jaxb.Context;
import org.docx4j.model.fields.merge.DataFieldName;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.PartName;
import org.docx4j.openpackaging.parts.WordprocessingML.HeaderPart;
import org.docx4j.openpackaging.parts.WordprocessingML.ImagePngPart;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.openpackaging.parts.relationships.RelationshipsPart.AddPartBehaviour;
import org.docx4j.relationships.Relationship;
import org.docx4j.wml.Body;
import org.docx4j.wml.CTBookmark;
import org.docx4j.wml.CTMarkupRange;
import org.docx4j.wml.ContentAccessor;
import org.docx4j.wml.Document;
import org.docx4j.wml.Hdr;
import org.docx4j.wml.ObjectFactory;
import org.docx4j.wml.P;
import org.docx4j.wml.SectPr;

/**
 * 
 * A watermark is set via the headers.
 * 
 *  This is different to w:document/w:background (see BackgroundImage sample for that,
 *  though this WatermarkPicture is probably what you want.)
 *  
 * @author jharrop
 */
public class DocXFormGenerator{
    public enum ResponseType {
        Boolean, String
    }
    public static class Response {
        public ResponseType type;
        public String value;
        public Response(ResponseType type, String value) {
            this.type = type;
            this.value = value;
        }
    }
    File DOCX_OUT; 
	
	//private static boolean DELETE_BOOKMARK = false;
	private boolean DELETE_BOOKMARK = false;
	
    //static ObjectFactory factory = Context.getWmlObjectFactory();
    ObjectFactory factory = Context.getWmlObjectFactory();
	//static File imageFile;
	File imageFile; 
    
    private byte[] image;    
    private WordprocessingMLPackage wordMLPackage;
    
    private static InputStream open(String file) {
        return DocXFormGenerator.class.getClassLoader().getResourceAsStream(DocXFormGenerator.class.getPackage().getName().replace('.', '/') + "/" + file);
    }
    
    // XXX refactor operation to remove need for "synchronized"
    public synchronized File doGeneration(Integer sectionNumber, LinkedList<Response> responses, String formType) throws Exception {
	addWaterMark(formType);
    	return populateForm(sectionNumber, responses);
    }
     
    
    public void addWaterMark(String formType) throws Exception {
    	
    	image = this.getImage();
    	
        try (InputStream template = open(formType.equals("InformedConsent") ? "ConsentForm.docx" : "HIPAAForm.docx")) {
            wordMLPackage = WordprocessingMLPackage.load(template);
        }
    	
    	// A watermark is defined in the headers, which are in turn set in sectPr
    	
			try {
				wordMLPackage.getMainDocumentPart().getContents().getBody().setSectPr(createSectPr());
			
					//inserts the answers contained in the map to the bookmark locations

				wordMLPackage.save(DOCX_OUT);
			} catch (Docx4JException e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
				throw new Exception("Had a problem retrieving the template document: " + e.getMessage());
			} catch (Exception e) {
				// TODO Auto-generated catch block
				//e.printStackTrace();
				throw new Exception("Had a problem, " + e.getMessage());
			} 
    }
    
    public File populateForm(Integer sectionNumber, LinkedList<Response> responses) throws Exception { //, ServletOutputStream sop
    	   	
    	Map<DataFieldName, String> map = new HashMap<>();//Response stored in a map to place in docx
        File file = File.createTempFile("gen", ".docx");
        file.deleteOnExit();
 		
        int counter = 1, j = 1, k = 1;
    	 
        for (Response response : responses) {
            System.out.println("User response = " + response.type.toString() + "," + (response.value != null ? response.value : "<null>") + " Counter = " + counter);
            
            if (response.type == ResponseType.Boolean) {
                System.out.println("Check for checkbox" + j + " Counter = " + counter);
                if (response.value != null)
                    xml(wordMLPackage, "Check" + j, response.value);
                j++;
            } else {
                if (response.value != null) {
                    map.put(new DataFieldName("Text" + k), response.value);
                    System.out.println("Docxgen Response " + k + " = " + response.value);
                }
                k++;
            }
            counter++;
        }
    	 	    			
	      		//System.out.println("Are we writing to console?");
	      		System.out.println("Title = " + wordMLPackage.getTitle());
	      		//Parts parts = wordMLPackage.getParts();
	      		//System.out.println(parts.toString());
	      		
	      		MainDocumentPart documentPart = wordMLPackage.getMainDocumentPart();
	      		org.docx4j.wml.Document wmlDocumentEl = (org.docx4j.wml.Document) documentPart.getJaxbElement();
	    		Body body = wmlDocumentEl.getBody();
	    		
	    		try {
					this.replaceBookmarkContents(body.getContent(), map);
				
	    		// background
	    		
	    		// save the docx...
	      		/*
	      		 * Make sure to change file path here //TODO
	      		 */
					
					/*
					File wordfile = new java.io.File(System.getProperty("user.dir") + "/103(After).docx");
					wordMLPackage.save(wordfile);
					OutputStream os = new FileOutputStream(wordfile);
					sop = (ServletOutputStream) os;
					*/
					//wordMLPackage.save(new java.io.File(System.getProperty("user.dir") + "/103(After).docx"));
										
					
					wordMLPackage.save(file);
					
	    		} catch (Exception e) {
					//e.printStackTrace();
					throw new Exception("Something bad happpend, " + e.getMessage()); // TODO Auto-generated catch block
				}
    	 
		return file;
    }
        
	private SectPr createSectPr() throws Exception {
		
		String openXML = "<w:sectPr xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\">"
				
				// Word adds the background image in each of 3 header parts
	            + "<w:headerReference r:id=\"" + createHeaderPart("even").getId() + "\" w:type=\"even\"/>"
	            + "<w:headerReference r:id=\"" + createHeaderPart("default").getId() + "\" w:type=\"default\"/>"
	            + "<w:headerReference r:id=\"" + createHeaderPart("first").getId() + "\" w:type=\"first\"/>"
	            
	            // Word adds empty footer parts when you create a watermark, but its not necessary
	            
//	            + "<w:footerReference r:id=\"rId9\" w:type=\"even\"/>"
//	            + "<w:footerReference r:id=\"rId10\" w:type=\"default\"/>"
//	            + "<w:footerReference r:id=\"rId12\" w:type=\"first\"/>"
	            
	            + "<w:pgSz w:h=\"15840\" w:w=\"12240\"/>"
	            + "<w:pgMar w:bottom=\"1440\" w:footer=\"708\" w:gutter=\"0\" w:header=\"708\" w:left=\"1440\" w:right=\"1440\" w:top=\"1440\"/>"
	            + "<w:cols w:space=\"708\"/>"
	            + "<w:docGrid w:linePitch=\"360\"/>"
	        +"</w:sectPr>";
	 
		return (SectPr)XmlUtils.unmarshalString(openXML);
	
	}

	private Relationship createHeaderPart(String nameSuffix) throws Exception {
		
		HeaderPart headerPart = new HeaderPart(new PartName("/word/header-" + nameSuffix + ".xml"));
		Relationship rel =  wordMLPackage.getMainDocumentPart().addTargetPart(headerPart);
		
		setHdr( headerPart);

		return rel;
	}
	
	private void setHdr(HeaderPart headerPart) throws Exception  {
		
		ImagePngPart imagePart = new ImagePngPart(new PartName("/media/background.png"));
		imagePart.setBinaryData(image);
		Relationship rel = headerPart.addTargetPart(imagePart, AddPartBehaviour.REUSE_EXISTING); // the one image is shared by the 3 header parts
		
		String openXML = "<w:hdr mc:Ignorable=\"w14 wp14\" xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:w=\"http://schemas.openxmlformats.org/wordprocessingml/2006/main\" xmlns:o=\"urn:schemas-microsoft-com:office:office\" xmlns:mc=\"http://schemas.openxmlformats.org/markup-compatibility/2006\" xmlns:r=\"http://schemas.openxmlformats.org/officeDocument/2006/relationships\">"
	            + "<w:p>"
	                + "<w:pPr>"
	                    + "<w:pStyle w:val=\"Header\"/>"
	                +"</w:pPr>"
	                + "<w:r>"
	                    + "<w:rPr>"
	                        + "<w:noProof/>"
	                    +"</w:rPr>"
	                    + "<w:pict>"
	                        + "<v:shapetype coordsize=\"21600,21600\" filled=\"f\" id=\"_x0000_t75\" o:preferrelative=\"t\" o:spt=\"75\" path=\"m@4@5l@4@11@9@11@9@5xe\" stroked=\"f\">"
	                            + "<v:stroke joinstyle=\"miter\"/>"
	                            + "<v:formulas>"
	                                + "<v:f eqn=\"if lineDrawn pixelLineWidth 0\"/>"
	                                + "<v:f eqn=\"sum @0 1 0\"/>"
	                                + "<v:f eqn=\"sum 0 0 @1\"/>"
	                                + "<v:f eqn=\"prod @2 1 2\"/>"
	                                + "<v:f eqn=\"prod @3 21600 pixelWidth\"/>"
	                                + "<v:f eqn=\"prod @3 21600 pixelHeight\"/>"
	                                + "<v:f eqn=\"sum @0 0 1\"/>"
	                                + "<v:f eqn=\"prod @6 1 2\"/>"
	                                + "<v:f eqn=\"prod @7 21600 pixelWidth\"/>"
	                                + "<v:f eqn=\"sum @8 21600 0\"/>"
	                                + "<v:f eqn=\"prod @7 21600 pixelHeight\"/>"
	                                + "<v:f eqn=\"sum @10 21600 0\"/>"
	                            +"</v:formulas>"
	                            + "<v:path gradientshapeok=\"t\" o:connecttype=\"rect\" o:extrusionok=\"f\"/>"
	                            + "<o:lock aspectratio=\"t\" v:ext=\"edit\"/>"
	                        +"</v:shapetype>"
	                        + "<v:shape id=\"WordPictureWatermark835936646\" o:allowincell=\"f\" o:spid=\"_x0000_s2050\" style=\"position:absolute;margin-left:0;margin-top:0;width:300%pt;height:300%pt;z-index:-251657216;mso-position-horizontal:center;mso-position-horizontal-relative:margin;mso-position-vertical:center;mso-position-vertical-relative:margin\" type=\"#_x0000_t75\">"
	                            + "<v:imagedata blacklevel=\"22938f\" gain=\"19661f\" o:title=\"docx4j-logo\" r:id=\"" + rel.getId() + "\"/>"
	                        +"</v:shape>"
	                    +"</w:pict>"
	                +"</w:r>"
	            +"</w:p>"
	        +"</w:hdr>";
		
			Hdr hdr = (Hdr)XmlUtils.unmarshalString(openXML);
			
			headerPart.setJaxbElement(hdr);

		}
	
	private byte[] getImage() throws IOException {
            boolean isAuthorized = false;

            DOCX_OUT = File.createTempFile("watermark", ".docx");
            DOCX_OUT.deleteOnExit();

            // Our utility method wants that as a byte array
            try (InputStream is = open((isAuthorized ? "" : "not") + "approved.png")) {
                ArrayList<Byte> a = new ArrayList<>();
                for (int b; (b = is.read()) != -1; )
                    a.add((byte)b);
                byte[] bytes = new byte[a.size()];
                for (int i = 0; i < bytes.length; i++)
                    bytes[i] = a.get(i);
                return bytes;
            }
	}
	
	/******************************************************************************************
	 * Checkbox logic method
	 ******************************************************************************************/
	/**
	 * @param wordMLPackage
	 * @param checkbox
	 * @param answer
	 * @throws JAXBException
	 * 
	 * Not currently being used (Alan)
	 */
	
	
	private static void xml (WordprocessingMLPackage wordMLPackage, String checkbox, String answer) throws JAXBException {
	    MainDocumentPart part = wordMLPackage.getMainDocumentPart();
	    String xml = XmlUtils.marshaltoString(part.getJaxbElement(), true);
		//JAXBContext jc = JAXBContext.newInstance(Document.class);  		//TODO Check if this fixes deprecated call
		//String xml = XmlUtils.marshaltoString(part.getJaxbElement(), jc); //TODO Check if this fixes deprecated call
	    
//
	    String xmlTest = "<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/>";
	    
	    // create XPath
	    XPathFactory factory = XPathFactory.newInstance();
	    XPath xPath = factory.newXPath();
	    
      
/*	    
	    if (xml.contains("<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData>")){
	    	System.out.println("found 1st format");
	    }else if (xml.contains("<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData>")){
	    	System.out.println("found 2nd format");
	    }else if (xml.contains("<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"0\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"0\"/></w:checkBox></w:ffData>")){
	    	System.out.println("found 3rd format");
	    }else{
	    	System.out.println("can't find any format");
	    }
*/	    
        //MainDocumentPart part = wordMLPackage.getMainDocumentPart();
        //String xml = XmlUtils.marshaltoString(part.getJaxbElement(), true);
        //Actual xml code that replaces the checked state of a checkbox.  Just need to match it to the corresponding book mark.
        //xml = xml.replaceAll("<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData>",
                                      //"<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"" + answer + "\"/><w:checked></w:checked></w:checkBox></w:ffData>");
        //xml = xml.replaceAll("<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData>",
          //           "<w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"" + answer + "\"/></w:checkBox></w:ffData>");
        if (xml.contains("<w:fldChar w:fldCharType=\"begin\"><w:ffData><w:name w:val=\"Check1\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData></w:fldChar>"))
                 {
                        System.out.println("Found checkbox!");
                 }
        
        xml = xml.replaceAll("<w:fldChar w:fldCharType=\"begin\"><w:ffData><w:name w:val=\"" + checkbox + "\"/><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"false\"/></w:checkBox></w:ffData></w:fldChar>", 
                 "<w:fldChar w:fldCharType=\"begin\"><w:ffData><w:name w:val=\"" + checkbox + "\" /><w:enabled/><w:calcOnExit w:val=\"false\"/><w:checkBox><w:sizeAuto/><w:default w:val=\"" + answer + "\"/></w:checkBox></w:ffData></w:fldChar>");
        
        Object obj = XmlUtils.unmarshalString(xml);
        part.setJaxbElement((Document) obj);

	    
	}
    
	/**************************************************************************
	 * Bookmark logic method
	 **************************************************************************/
	public  void replaceBookmarkContents(List<Object> paragraphs,  Map<DataFieldName, String> data) throws Exception {

		RangeFinder rt = new RangeFinder("CTBookmark", "CTMarkupRange");
		new TraversalUtil(paragraphs, rt);
		
		for (CTBookmark bm : rt.getStarts()) {
			
			// do we have data for this one?
			if (bm.getName()==null) continue;
			String value = data.get(new DataFieldName(bm.getName()));
			if (value==null) continue;
				
			try {
				// Can't just remove the object from the parent,
				// since in the parent, it may be wrapped in a JAXBElement
				List<Object> theList = null;
				if (bm.getParent() instanceof P) {
					theList = ((ContentAccessor)(bm.getParent())).getContent();
				} else {
					continue;
				}

				int rangeStart = -1;
				int rangeEnd=-1;
				int i = 0;
				for (Object ox : theList) {
					Object listEntry = XmlUtils.unwrap(ox); 
					if (listEntry.equals(bm)) {
						if (DELETE_BOOKMARK) {
							rangeStart=i;
						} else {
							rangeStart=i+1;							
						}
					} else if (listEntry instanceof  CTMarkupRange) {
						if ( ((CTMarkupRange)listEntry).getId().equals(bm.getId())) {
							if (DELETE_BOOKMARK) {
								rangeEnd=i;
							} else {
								rangeEnd=i-1;							
							}
							break;
						}
					}
					i++;
				}
				
				if (rangeStart>0 && rangeEnd>rangeStart) {
					
					// Delete the bookmark range
					for (int j =rangeEnd; j>=rangeStart; j--) {
						theList.remove(j);
					}
					
					// now add a run
					org.docx4j.wml.R  run = factory.createR();
					org.docx4j.wml.Text  t = factory.createText();
					org.docx4j.wml.CTBookmark bookmark = factory.createCTBookmark();
					run.getContent().add(t);		
					t.setValue(value);
					
					theList.add(rangeStart, run);
				}
				
			} catch (ClassCastException cce) {
				System.out.println(cce.getMessage());
				System.out.println(cce.getStackTrace());
			}
		}	
	}
}