/*
*  E1.1.0 RAMS Increment 1 Patch Code
*  January 2015
*  Department of Veterans Affairs/Booz Allen Hamilton
*/

package gov.va.rams.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

/**
 *
 * @author Brian
 */
@Entity
@Table(name = "Questions_Responses")
@XmlRootElement
public class QuestionResponse implements Serializable {
    private static final long serialVersionUID = 1L;

    @Id
    @Basic(optional = false)
    @Column(name = "QuestionsResponsesID")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer questionsResponsesID;
    @Column(name = "CheckboxIndex")
    private Integer checkboxIndex;
    @Size(max = 1073741823)
    @Column(name = "Response")
    private String response;
    @Basic(optional = false)
    @NotNull
    @Column(name = "UpdatedAt")
    @Temporal(TemporalType.TIMESTAMP)
    private Date updatedAt;
    private Integer answersID;
    private Integer questionsID;
    @JoinColumn(name = "StudyStatusID", referencedColumnName = "StudyStatusID")
    @ManyToOne(optional = false)
    private StudyStatus studyStatus;
    
    public static class Key extends ArrayList<Integer> {
        public Key(QuestionResponse qr) {
            this(qr.questionsID, qr.getCheckboxIndex());
        }

        public Key(Integer questionsID, Integer checkboxIndex) {
            super(2);
            add(questionsID);
            add(checkboxIndex);
        }
        
        public Key(Integer questionsID) {
            this(questionsID, null);
        }
    }

    public QuestionResponse() {
    }

    public boolean isUnset() {
        return answersID == null && response == null;
    }

    @XmlTransient
    public Integer getQuestionsResponsesID() {
        return questionsResponsesID;
    }

    public void setQuestionsResponsesID(Integer questionsResponsesID) {
        this.questionsResponsesID = questionsResponsesID;
    }
    
    public Integer getCheckboxIndex() {
        return checkboxIndex;
    }

    public void setCheckboxIndex(Integer checkboxIndex) {
        this.checkboxIndex = checkboxIndex;
    }

    public String getResponse() {
        return response;
    }

    public void setResponse(String response) {
        this.response = response;
    }

    public Date getUpdatedAt() {
        return updatedAt;
    }

    public void setUpdatedAt(Date updatedAt) {
        this.updatedAt = updatedAt;
    }

    public Integer getAnswersID() {
        return answersID;
    }

    public void setAnswersID(Integer answersID) {
        this.answersID = answersID;
    }

    public Integer getQuestionsID() {
        return questionsID;
    }

    public void setQuestionsID(Integer questionsID) {
        this.questionsID = questionsID;
    }
    
    @XmlTransient
    public StudyStatus getStudyStatus() {
        return studyStatus;
    }

    public void setStudyStatus(StudyStatus studyStatus) {
        this.studyStatus = studyStatus;
    }


    @Override
    public int hashCode() {
        int hash = 0;
        hash += (questionsResponsesID != null ? questionsResponsesID.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof QuestionResponse)) {
            return false;
        }
        QuestionResponse other = (QuestionResponse) object;
        if ((this.questionsResponsesID == null && other.questionsResponsesID != null) || (this.questionsResponsesID != null && !this.questionsResponsesID.equals(other.questionsResponsesID))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "gov.va.rams.entities.QuestionResponse[ questionsResponsesID=" + questionsResponsesID + " ]";
    }

    public static HashMap<QuestionResponse.Key, QuestionResponse> rollup(List<QuestionResponse> responsesByStage) {
        HashMap<QuestionResponse.Key, QuestionResponse> rollup = new HashMap<>();
        for (QuestionResponse qr : responsesByStage) {
            if (qr.isUnset()) {
                rollup.remove(new QuestionResponse.Key(qr));
            } else {
                rollup.put(new QuestionResponse.Key(qr), qr);
            }
        }
        return rollup;
    }
}
