﻿/*
*  E1.1.0 RAMS Increment 1 Patch Code
*  January 2015
*  Department of Veteran’s Affairs/Booz Allen Hamilton
*/


package gov.va.rams.service;

import gov.va.rams.entities.Answer;
import gov.va.rams.entities.ContainedForm;
import gov.va.rams.entities.Form;
import gov.va.rams.entities.FormQuestion;
import gov.va.rams.entities.FormVersion;
import gov.va.rams.entities.Question;
import gov.va.rams.entities.QuestionResponse;
import gov.va.rams.entities.StudyStatus;
import gov.va.rams.service.elements.FormStatus;
import gov.va.rams.service.elements.FormValidation;
import gov.va.rams.service.elements.FormValidationReport;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.ejb.Singleton;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

/**
 * Calculate and cache completion status of the form tree.
 * We are also responsible for cleaning up excess responses that
 * are orphaned when required predecessor answers become unset.
 */
@Singleton
public class ResponseValidator {
    @PersistenceContext(unitName = "RAMSPU")
    private EntityManager em;
    
    private HashMap<StudyStatus, FormValidationReport> cache = new HashMap<>();

    public FormValidationReport getFormValidationReport(StudyStatus status) {
        return cache.get(status);
    }

    public FormValidationReport updateFormValidationReport(StudyStatus status, HashMap<QuestionResponse.Key, QuestionResponse> rollup, boolean cleanup) {
        FormValidationReport report = new FormValidationReport();
        
    	//create collection for lookup with all answered question id's
    	HashSet<Integer> completeQuestionResponseCollection = new HashSet<Integer>();
    	HashSet<Integer> completeAnswersCollection = new HashSet<Integer>();
		Iterator<?> it = rollup.entrySet().iterator();
		while( it.hasNext() ){
		     Entry<?, ?> entry = (Entry<?, ?>) it.next();
		     QuestionResponse qr = (QuestionResponse) entry.getValue();
		     //System.out.println( String.format( "(%s, %s, %s)", entry.getKey(),  entry.getValue(),  qr.getQuestionsID()) );
		     completeQuestionResponseCollection.add(qr.getQuestionsID());
		     completeAnswersCollection.add(qr.getAnswersID()); //TODO can probably pair this down
		}
		
		//perform jpa query to get root form
		Form node = em.find(FormVersion.class, status.getStudy().getFormVersionsID()).getRootForm();
		
		System.out.println("Anwser Collection Size = " + completeAnswersCollection.size());
/*		
		for (ContainedForm cf : node.getContainedFormCollection()){
			Form pf = cf.getContainerForm();
			System.out.println(pf.getNarrative());
			List<?> qc = (List<?>) pf.getFormQuestionCollection();
			for(int i=0; i<qc.size(); i++){
				FormQuestion fq = (FormQuestion) qc.get(i);
				System.out.println("Form Questions Predecessor Answers = " + fq.getRequiredPredecessorAnswersID());
			}
			
		}	
*/
		
        updateValidationReport(report, node, completeQuestionResponseCollection, completeAnswersCollection, cleanup ? em.find(StudyStatus.class, status.getStudyStatusID()) : null, cleanup ? rollup : null); //completeAnswersCollection
        cache.put(status, report);
        return report;
    }
    
    //validation    
    private void updateValidationReport(FormValidationReport report, Form node, HashSet<Integer> cQRC, HashSet<Integer> cAC, StudyStatus cleanup, HashMap<QuestionResponse.Key, QuestionResponse> rollup) {
    	if (cleanup != null)
            cleanup.setUpdatedAt(new Date());
        Set<Integer> disabledQuestionAnswers = new HashSet<>();
    	ArrayList<FormValidation> fva = new ArrayList<FormValidation>();
//    	System.out.println("contained form collection size = " + node.getContainedFormCollection().size());
//    	System.out.println("array size = " + fva.size());
    	
    	//iterate over contained form collection
        for (ContainedForm cf : node.getContainedFormCollection()){
//        	System.out.println("Iteration " + c );
        	try {
        				
				Form f = cf.getContainedForm();
				List<?> qc = (List<?>) f.getFormQuestionCollection();
				System.out.println("Form narrative = " + f.getNarrative());
				
				int completenessCount = 0, ignoredCount = 0; //added items to ignore
				for(int i=0; i<qc.size(); i++){
                                        boolean nullify = false;
					FormQuestion fq = (FormQuestion) qc.get(i);
					Question q = fq.getQuestion();
					if (fq.getRequiredPredecessorAnswersID() != null) { // There is a required predecessor answer:
                            if (disabledQuestionAnswers.contains(fq.getRequiredPredecessorAnswersID())) { // but the required answer's question is disabled
                                ignoredCount++; // so treat it as not required
                                nullify = true;
                                for (Answer a : q.getAnswerCollection()) // recursively disable later questions that depend on this one's answers
                                    disabledQuestionAnswers.add(a.getAnswersID());
                            } else if (!cAC.contains(fq.getRequiredPredecessorAnswersID()) // We are only incomplete if the required answer is set and we don't have a response for this question.
                            		|| cQRC.contains(q.getQuestionsID())){
                            	ignoredCount++;
                                nullify = true;
                            }
					}else if(cQRC.contains(q.getQuestionsID())) { // No required predecessor.
						completenessCount++;
					}
				                                
                                    // Are we being requested to clean up a StudyStatus?
                                    if (cleanup != null && nullify && cQRC.contains(q.getQuestionsID())) {
                                        if (q.getResponseFormat().getName().equals("checkbox")) { // multi-answer mode
                                            for (int answer = 1; answer <= q.getAnswerCollection().size(); answer++) {
                                                QuestionResponse qr = rollup.get(new QuestionResponse.Key(q.getQuestionsID(), answer));
                                                if (qr == null) {
                                                    // ignore
                                                } else if (qr.getStudyStatus().equals(cleanup)) { // current
                                                    for (QuestionResponse search : cleanup.getQuestionResponseCollection()) {
                                                        if (search.equals(qr)) {
                                                            nullout(search);
                                                            search.setUpdatedAt(cleanup.getUpdatedAt());
                                                            break;
                                                        }
                                                    }
                                                } else { // inherited
                                                    qr = new QuestionResponse();
                                                    qr.setQuestionsID(q.getQuestionsID());
                                                    qr.setCheckboxIndex(answer);
                                                    qr.setStudyStatus(cleanup);
                                                    qr.setUpdatedAt(cleanup.getUpdatedAt());
                                                    cleanup.getQuestionResponseCollection().add(qr);
                                                }
                                            }
                                        } else { // single-answer mode
                                            QuestionResponse qr = rollup.get(new QuestionResponse.Key(q.getQuestionsID()));
                                            if (qr.getStudyStatus().equals(cleanup)) { // current
                                                for (QuestionResponse search : cleanup.getQuestionResponseCollection()) {
                                                    if (search.equals(qr)) {
                                                        nullout(search);
                                                        search.setUpdatedAt(cleanup.getUpdatedAt());
                                                        break;
                                                    }
                                                }
                                            } else { // inherited
                                                qr = new QuestionResponse();
                                                qr.setQuestionsID(q.getQuestionsID());
                                                qr.setStudyStatus(cleanup);
                                                qr.setUpdatedAt(cleanup.getUpdatedAt());
                                                cleanup.getQuestionResponseCollection().add(qr);
                                            }
                                        }
                                    }
				}
				
				//System.out.println("Required PredecessorAnswersID = " + cf.getRequiredPredecessorAnswersID());
				int requiredPredForm = -1;
				
				if (cf.getRequiredPredecessorAnswersID()!= null){
					System.out.println("Required Form PredecessorAnswersID = " + cf.getRequiredPredecessorAnswersID());
					requiredPredForm = cf.getRequiredPredecessorAnswersID();
				}

				if ( requiredPredForm >= 0 && !cAC.contains(requiredPredForm)){ //form required (search answers)
					System.out.println("No Form PredecessorAnswersID");
					fva.add(new FormValidation(f, FormStatus.NOT_APPLICABLE));
                                        for (FormQuestion fq : f.getFormQuestionCollection()) // Disabled section, so all contained answers count as disabled.
                                            for (Answer a : fq.getQuestion().getAnswerCollection())
                                                disabledQuestionAnswers.add(a.getAnswersID());
        		}else if (completenessCount == 0){
					System.out.println("Response size is zero");
					fva.add(new FormValidation(f, FormStatus.EMPTY));
				}else if (completenessCount + ignoredCount < qc.size()){
					System.out.println("Response size is " + completenessCount);
					fva.add(new FormValidation(f, FormStatus.INCOMPLETE));
				}else if (completenessCount + ignoredCount == qc.size()){
					System.out.println("response complete");
					fva.add(new FormValidation(f, FormStatus.COMPLETE));	
				}
				
				report.setFormStatusCollection(fva);
			} catch (Exception e) {
				System.out.println("Error : " + e.getMessage());
			}
        	  
//        	c++;
       	       	
        }
        	
    }

    private void nullout(QuestionResponse qr) {
        qr.setAnswersID(null);
        qr.setResponse(null);
    }
}
