/***************************************************
E1.1.0 RAM Increment 1 Patch Code
January 2015
Department of Veteran's Affairs/Booz Allen Hamilton
dynamicGeneration.js
***************************************************/

//template to hold overall study with all display info
var study;

//template to keep track of which questions are answered in a section
var changedQuestions;

//template to keep track of all the current users
var users;

//template to hold all responses from a user
var rollup;

//template to keep track of if the study is in draft or in review
var recentStage;

//template for the study title
var studyTitle;

//study title's id
var studyTitleID = "0";

//template for the study ID
var assignedID = "1";

//template for map between parent/child questions
var dependency = new Hashtable();

//holds the PI and CO-PI as they're filled out
var studyPI = new Hashtable();

//Set of all answer IDs that are in the rollup
var rollupAnswers = new HashSet();

//Linking in the sql database
var restPath = "http://vaphiramapp20.domain:8080/RAMS-0.0.1/";
//Linking in the sharepoint site
var siteUrl = "http://vaww.RAM.msp.domain/sites/IRBApplications/";

//template for the user.
var currentUser={
    "displayName": "initialUser",
    "loginName": null,
    "permanentLoginIdentifier": null,
    "usersID": null
};

//Inital function called by HTML. Sets up baseline information needed.
function process(){
    //Grabs username and study ID
    getCookie();

    //Getting the most recent stage to use later
    if(assignedID != "1"){
        studyResponse = sendREST("GET", "study/"+assignedID, null, false);
        if(!("length" in studyResponse.study.studyStatusCollection))
            studyResponse.study.studyStatusCollection = [studyResponse.study.studyStatusCollection];
        recentStage = studyResponse.study.studyStatusCollection.length;
    }else{
        recentStage = parseInt(assignedID);
    }

    //most recent forms
    recentForms = sendREST("GET", "forms", null, false);

    //getting all initial questions
    study = sendREST("GET", "forms/"+recentForms.formVersion[0].formVersionsID, null, false);

    //building section A
    sectionBuilder(0);
};

//Getting user's responses from the database and updating the local version of
//the user's responses, called rollupAnswers
function getRollup(){

    //all user's responses that are in the database
    if(assignedID!="1"){
        //When the assignedID is set to 1, we know that there is no data in the database for a study
        rollup = sendREST("GET", "study/"+assignedID+"/responses/"+recentStage+"/rollup", null, false);

        //formatting the recieved JSON
        if(!("responses" in rollup.questionResponseUpdates)){
            rollup.questionResponseUpdates.responses = [];
        }else if(!("length" in rollup.questionResponseUpdates.responses)){
            rollup.questionResponseUpdates.responses = [rollup.questionResponseUpdates.responses];
        }

        //below we are statically grabbing the study title, PI, and CO-PI based on generated position
        studyTitleID = study.form.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;

        for(var j=0; j<rollup.questionResponseUpdates.responses.length; j++){
            if(rollup.questionResponseUpdates.responses[j].answersID){
                rollupAnswers.add(rollup.questionResponseUpdates.responses[j].answersID);
            }

            //setting study title
            if(rollup.questionResponseUpdates.responses[j].questionsID == studyTitleID)
                studyTitle = rollup.questionResponseUpdates.responses[j].response;

        }
    }else{
        //setting an empty rollup
        rollup = {};
    }
};

//getting the user and study id that was passed to the application
function getCookie(){
    var userCookie, studyCookie, splitURL;

    //finding the user information in the database 
    users = sendREST("GET", "user", null, false);
        
    splitURL = window.location.href.split("?");

    //study id that was passed
    assignedID = splitURL[1];

    //user that was passed
    currentUser.displayName = splitURL[2];

    //looking the user up in the database
    for(var i=0; i<users.user.length; i++){
        if(users.user[i].displayName == currentUser.displayName)
            currentUser.loginName = users.user[i].loginName;
    }
};

//sets up the structure of the question/sidebar section and 
//attaches to the proper area in webpage.
function sectionBuilder(targetSection){
    rollupAnswers = new HashSet();
    
    //updating the user's responses every time a new
    //section is built
    getRollup();

    //removing any trace of the last section that was built
    $('div.row').remove();
    var section, row, column, sideColumn;
    
    //setting up the initial structure of page
    row = document.createElement("div");
    row.setAttribute("class", "row");
    $('div#insert').append(row);

    column = document.createElement("div");
    column.setAttribute("class", "col-xs-9");
    $('div.row').append(column);

    columnBuilder(targetSection);

    sideColumn = document.createElement("div");
    sideColumn.setAttribute("class", "col-xs-3 toolbar");
    $('div.row').append(sideColumn);

    sideColumnBuilder(targetSection);
};

//sets up the structure of the question area; includes header,
//questions, and button dock (progress buttons)
function columnBuilder(targetSection){
    var pageHeader, formContent, progressContainer;

    section = study.form.containedFormCollection[targetSection];

    //page header
    pageHeader = document.createElement("div");
    pageHeader.setAttribute("class", "page-header");
    $('div.col-xs-9').append(pageHeader);

    headerBuilder(targetSection);

    //questions
    formContent = document.createElement("div");
    formContent.setAttribute("class", "formContent");
    $('div.col-xs-9').append(formContent);

    if((targetSection == 1) || (targetSection == 5) || (targetSection == 6) || (targetSection == 7) || (targetSection == 11))
        disclaimerBuilder(targetSection);

    questionBuilder(section);
    
    //progress buttons
    progressContainer = document.createElement("div");
    progressContainer.setAttribute("class", "progress-container");
    $('div.col-xs-9').append(progressContainer);

    progressBuilder(targetSection);
};

//Sets up the structure of the sidebar; includes section buttons, print and upload buttons,
//and review and submit buttons.
function sideColumnBuilder(targetSection){
    var actionBar, wsBar;

    //action bar
    actionBar = document.createElement("div");
    actionBar.setAttribute("class", "actionbar");
    $('div.col-xs-3').append(actionBar);

    actionBarBuilder(targetSection);

    //well sidebar
    wsBar = document.createElement("div");
    wsBar.setAttribute("class", "well sidebar");
    $('div.col-xs-3').append(wsBar);

    wellSidebarBuilder(targetSection);
};

//creating the disclaimer that is seen at the top of certain page
function disclaimerBuilder(targetSection){
    var cont, close, hide, closer, paragraph;
    cont = document.createElement("div");
    cont.setAttribute("class", "well notificationArea alert-dismissible fade in");
    cont.setAttribute("role", "Alert");

    close = document.createElement("button");
    close.setAttribute("data-dismiss", "alert");
    close.setAttribute("class", "close");
    close.setAttribute("type", "button");

    hide = document.createElement("span");
    hide.setAttribute("aria-hidden", "true");
    var text = document.createTextNode("x");
    hide.appendChild(text);

    closer = document.createElement("span");
    closer.setAttribute("class", "sr-only");
    var text = document.createTextNode("Close")
    closer.appendChild(text);

    close.appendChild(hide);
    close.appendChild(closer);

    paragraph = document.createElement("p");

    if(targetSection == 1){
        var text = document.createTextNode("Disclaimer: Answers to some questions in this section will require your answer to additional questions in subsequent sections of the IRB Application.");
    }else if(targetSection == 5){
        var text = document.createTextNode("Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.");
    }else if(targetSection == 6){
        var text = document.createTextNode("Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.");
    }else if(targetSection == 7){
        var text = document.createTextNode("Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.");
    }else if(targetSection == 11){
        var text = document.createTextNode("Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.");
    }

    paragraph.appendChild(text);

    cont.appendChild(close);
    cont.appendChild(paragraph);
    $('.formContent').append(cont);
};

//builds the header that displays information about a study and section.
function headerBuilder(targetSection){
    var appTitle, id, section, sectionContainer, genInfo, comp;

    //Study title
    appTitle = document.createElement("h2");
    appTitle.setAttribute("class", "applicationTitle");
    
    //Sets up the default study title in the header
    if(studyTitle === undefined){
        var text = document.createTextNode("New IRB Application");
    }else{
        var text = document.createTextNode(studyTitle);
    }
    appTitle.appendChild(text);

    //Study ID
    id = document.createElement("small");
    
    //Sets up the default study ID in the header
    if(studyTitle === undefined){
        var text = document.createTextNode(" ID: TBD");
    }else{
        var text = document.createTextNode(" ID: " + assignedID);
    }
    id.appendChild(text);
    appTitle.appendChild(id);

    $('div.page-header').append(appTitle);

    //Current Section Display
    section = document.createElement("h3");
    section.setAttribute("class", "sectionNumber");
    
    var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    var text = document.createTextNode("Section " + sectionTitle.charAt(targetSection));
    section.appendChild(text);
    
    $('div.page-header').append(section);
    
    //General Information Display
    sectionContainer = document.createElement("div");
    sectionContainer.setAttribute("class", "sectionContainer");

    genInfo = document.createElement("h5");
    genInfo.setAttribute("class", "sectionTitle");

    var text = document.createTextNode(study.form.containedFormCollection[targetSection].containedForm.title);
    genInfo.appendChild(text);
    
    sectionContainer.appendChild(genInfo);
    
    comp = document.createElement("h5");
    comp.setAttribute("class", "sectionLocation");
    
    //Setting up a placeholder for completion status
    var text = document.createTextNode("Completed: 0/11");
    comp.appendChild(text);

    sectionContainer.appendChild(comp);

    $('div.page-header').append(sectionContainer);
};

//Creates a general warning to display. Takes in a message and a warning type,
//which will move the user to the progress bar when called.
function insertWarning(message, type){
    var warnCont, bolded;
    
    //Removing any traces of previous warnings.
    $('.alert').remove();

    warnCont = document.createElement("div");
    warnCont.setAttribute("class", "alert "+type);
    warnCont.setAttribute("role", "alert");

    //Creating different content based on what is requested
    bolded = document.createElement("strong");
    if(type == "alert-warning"){
        var text = document.createTextNode("Warning! ");
    }else if(type == "alert-success"){
        var text = document.createTextNode("Congratulations! ");
    }
    bolded.appendChild(text);
    warnCont.appendChild(bolded);

    //linking everything to the page
    var text = document.createTextNode(message);
    warnCont.appendChild(text);
    
    //JQUERY moving page to the generated warning
    $('div.progress-container').prepend(warnCont);
    $('html, body').animate({
	    scrollTop: $(document).height()
	}, 1000);
};

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, sync){
    var xmlResponse;
    
    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
	xmlResponse = new XMLHttpRequest();
    }else{
	xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
    	xmlResponse.open(httpMethod, restPath.concat(endpoint), sync);
        xmlResponse.setRequestHeader("Cache-Control", "no-cache");
    	xmlResponse.setRequestHeader("Content-Type", "application/json");
    	xmlResponse.setRequestHeader("Accept", "application/json");
    	
    	if((httpMethod == "PUT")||(httpMethod == "DELETE")){
    	    xmlResponse.setRequestHeader("sm_user", data);
    	    xmlResponse.send(null);
    	    
            //204 is the response status indicating success, while
            //403 indicates failure
            if(xmlResponse.status === 204){
                return true;
    	    }else if(xmlResponse.status === 403){
                return false;
    	    }
    	}else if(httpMethod == "POST"){
    	    try{
                //sending data and alerting the user
        		xmlResponse.send(JSON.stringify(data));
                insertWarning('You have successfully saved your responses', 'alert-success');
    	    }catch(e){
                alert(e.toString());
    	    }
    	}else if(httpMethod == "GET"){
            //retrieving the needed data
    	    xmlResponse.onreadystatechange = function(){
                if(xmlResponse.readyState === 4){
                    if(xmlResponse.status === 200){	
                    	try{
                    	     xmlResponse = JSON.parse(xmlResponse.responseText);
                    	}catch(e){
                    	    alert(e.toString());
                    	}	
                    }else{
                        if((xmlResponse.status !== 204) && (xmlResponse.status !== 1223))
                    	 alert(xmlResponse.status);
                    }
                }
    	    };
    	    xmlResponse.send(null);
            //passing the needed data to the function that is requesting it
    	    return xmlResponse;
    	}

    }catch(e){
	   alert(e.toString());
    }
};

//Takes the response from the database and builds the questions displayed
//in the application.
function questionBuilder(section){
    //setting up a fresh set to keep track of questions that have been
    //changed while the user is in a particular section.
    changedQuestions = new HashSet();

    //accounting for the JSON format that is passed by jaxb
    if(!("length" in section.containedForm.formQuestionCollection))
	   section.containedForm.formQuestionCollection = [section.containedForm.formQuestionCollection];
    
    //filtering through each of the questions in a section to build them individually
    for(var i=0; i<section.containedForm.formQuestionCollection.length; i++){
	var form, formGroup, textLabel, text, input, toolSpan, field, legend;
	var formQuestion = section.containedForm.formQuestionCollection[i];

	form = document.createElement("form");
	
	//div containing complete questions
	formGroup = document.createElement("div");
	formGroup.setAttribute("class", "form-group");
		
	//question text. We see the introdocution of QID_ (Question IDs) and later
    //we will see AID_ (Answer IDs). These have been put in to comply 508 standards that 
    //input id attributes must start with a letter.
	textLabel = document.createElement("label");
	textLabel.setAttribute("for", "QID_" + formQuestion.question.questionsID);
	
	text = document.createTextNode(formQuestion.question.questionHyperText);
	textLabel.appendChild(text);

	//tooltip logic
	if(formQuestion.question.helpHyperText){
	    toolSpan = document.createElement("span");
	    toolSpan.setAttribute("data-toggle", formQuestion.question.helpHyperText);
	    toolSpan.setAttribute("data-placement", "right");
	    toolSpan.setAttribute("title", formQuestion.question.helpHyperText);
	    toolSpan.setAttribute("class", "glyphicon glyphicon-question-sign");
        toolSpan.style.cursor = "help";
	
	    textLabel.appendChild(toolSpan);
	}
	
	//Building the questions based on their type: radio, checkbox, select, textarea, file button, etc
	if((formQuestion.question.responseFormat.name == "radio") || (formQuestion.question.responseFormat.name == "checkbox")){
        //question input for radios and checkboxes
        field = document.createElement("fieldset");
        legend = document.createElement("legend");
        legend.appendChild(textLabel);
        field.appendChild(legend);

        if(!("length" in formQuestion.question.answerCollection))
		  formQuestion.question.answerCollection = [formQuestion.question.answerCollection];
	    
        //building the individual radio/checkbox buttons one by one.
        for(var j=0; j<formQuestion.question.answerCollection.length; j++){
    		var answerLabel = document.createElement("label");
    			
    		//radio button formatting based on the length of text associated with the radio button.
    		if((formQuestion.question.responseFormat.name == "radio") && (formQuestion.question.answerCollection[j].value.length < 6)){
    		    answerLabel.setAttribute("class", "radio-inline");
    		}else if((formQuestion.question.responseFormat.name == "radio") && (formQuestion.question.answerCollection[j].value.length >= 6)){
    		    answerLabel.setAttribute("class", "radio");
    		}else if(formQuestion.question.responseFormat.name == "checkbox"){
    		    answerLabel.setAttribute("class", "checkbox");
    		}
            answerLabel.setAttribute("for", "AID_" + formQuestion.question.answerCollection[j].answersID);
    		
            //putting together general structure
    		var input = document.createElement("input");
    		input.setAttribute("type", formQuestion.question.responseFormat.name);
    		input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
    		input.setAttribute("id", "AID_" + formQuestion.question.answerCollection[j].answersID);

            //disabling questions if the study is in review
            if(recentStage == "2"){
                input.setAttribute("disabled", true);
            }

            //Specifying the method to notify which questions are changed. This method helps group radio buttons
            //together, while allowing checkboxes to be saved individually. 
    		if(formQuestion.question.responseFormat.name == "checkbox"){
    		    input.setAttribute("title", formQuestion.question.answerCollection[j].sequencePosition);
    		    input.setAttribute("onchange", "Function(changedQuestions.add("+formQuestion.question.answerCollection[j].answersID+"))");
    		}else{
    		    input.setAttribute("onchange", "Function(changedQuestions.add("+formQuestion.question.questionsID+"))");
    		}
    		if((rollup.questionResponseUpdates === undefined) || (rollup.questionResponseUpdates.responses === undefined)){
    		    
    		}else{
    		    //looking for user's responses that have been grabbed from the database and updating questions.
    		    for(var x=0; x<rollup.questionResponseUpdates.responses.length; x++){
        			if((rollup.questionResponseUpdates.responses[x].questionsID == formQuestion.question.questionsID) && (rollup.questionResponseUpdates.responses[x].answersID) == formQuestion.question.answerCollection[j].answersID){
        			    input.checked = true;
        			}
    		    }
    		}
    		
    		var value = document.createTextNode(formQuestion.question.answerCollection[j].value);
    		
    		answerLabel.appendChild(input);
    		answerLabel.appendChild(value);
    		
    		field.appendChild(answerLabel);
            formGroup.appendChild(field);
            form.appendChild(formGroup);
    		$('div.formContent').append(form);
	    }
	}else if(formQuestion.question.responseFormat.name == "select"){
	    formGroup.appendChild(textLabel);
	    form.setAttribute("role", "form")
	    form.appendChild(formGroup);
	    
        //setting up the general structure of a drop down
	    input = document.createElement("select");
	    var inputOption = document.createElement("option");
	    inputOption.setAttribute("value", "none");
	    var text = document.createTextNode("none");
	    inputOption.appendChild(text);
	    input.appendChild(inputOption);

        //populating drop down based on users grabbed from the database
        for(var j=0; j<users.user.length; j++){
            inputOption = document.createElement("option");
            inputOption.setAttribute("value", users.user[j].loginName);
            text = document.createTextNode(users.user[j].loginName);
            inputOption.appendChild(text);
            input.appendChild(inputOption);
        }

	    input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
	    input.setAttribute("class", "form-control normal select");
        input.setAttribute("onclick", "changedQuestions.add("+formQuestion.question.questionsID+")");
        
        //disabling question if the study is in review
        if(recentStage == "2"){
            input.setAttribute("disabled", true);
        }

	    formGroup.appendChild(input);
	    $('div.formContent').append(form);

        if((rollup.questionResponseUpdates === undefined) || (rollup.questionResponseUpdates.responses === undefined)){
        }else{
            //changing which option in the dropdown is initially selected
            for(var x=0; x<rollup.questionResponseUpdates.responses.length; x++){
                if(rollup.questionResponseUpdates.responses[x].questionsID == formQuestion.question.questionsID){
                input.value = rollup.questionResponseUpdates.responses[x].response;
                
                studyPI.put(formQuestion.question.questionsID, rollup.questionResponseUpdates.responses[x].response);
                }
            }   
        }

	}else if((formQuestion.question.responseFormat.name == "button")&&(recentStage != "2")){

	    var uploadInput, uploadLink, uploadIcon;
        var noFile = true;
	    
	    formGroup.appendChild(textLabel);
	    
        //setting up the general structure for a file upload button
	    form.setAttribute("role", "form");

        //setting up the upload button
        uploadInput = document.createElement("input");
        uploadInput.setAttribute("id", "upload");
        uploadInput.setAttribute("name", "QID_" + formQuestion.question.questionsID);
        uploadInput.setAttribute("type", "file");

        //attaching the upload button
        formGroup.appendChild(uploadInput);
	    
        if((rollup.questionResponseUpdates === undefined) || (rollup.questionResponseUpdates.responses === undefined)){
        }else{
        //looking for responses to populate the textarea with
            for(var x=0; x<rollup.questionResponseUpdates.responses.length; x++){
                if(rollup.questionResponseUpdates.responses[x].questionsID == formQuestion.question.questionsID){
                var text = document.createTextNode("Uploaded File: " + rollup.questionResponseUpdates.responses[x].response);
                formGroup.appendChild(text);
                uploadInput.setAttribute("disabled", "disabled");
                }
            }
        }

	    form.appendChild(formGroup);
	    $('div.formContent').append(form);
        
	}else{
	    //question input for everything else, which should cover textares. 
	    formGroup.appendChild(textLabel);
	    form.setAttribute("role", "form");
	    form.appendChild(formGroup);

        //setting up the general structure for a textarea
        if(formQuestion.question.responseFormat.name == "textarea"){
            input = document.createElement("textarea");
            input.setAttribute("rows", "10");
        }else{
            input = document.createElement("input");
            input.setAttribute("type", formQuestion.question.responseFormat.name);
        }

        //setting up the general structure for a textarea
	    input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
	    input.setAttribute("class", "form-control normal");
	    input.setAttribute("onchange", "Function(changedQuestions.add("+formQuestion.question.questionsID+"))");
	    
        //making sure the textarea is disabled if the application is in review
        if(recentStage == "2"){
            input.setAttribute("disabled", true);
        }

	    if((rollup.questionResponseUpdates === undefined) || (rollup.questionResponseUpdates.responses === undefined)){
	    }else{
		//looking for responses to populate the textarea with
    		for(var x=0; x<rollup.questionResponseUpdates.responses.length; x++){
    		    if(rollup.questionResponseUpdates.responses[x].questionsID == formQuestion.question.questionsID){
    			input.value = rollup.questionResponseUpdates.responses[x].response;
    		    }
    		}
	    }

	    formGroup.appendChild(input);
	    $('div.formContent').append(form);
	    
	}

	//checking to see if the question has a dependency on another answer
	if((formQuestion.requiredPredecessorAnswersID)&&(recentStage != "2")){	   
        questionRequirement(formQuestion, input);
	}

    //making sure the question is completely disabled if the application is in review
    if(recentStage == "2"){
        input.setAttribute("disabled", true);
    }

    }
};

//checks to see if there are any unsaved questions within
//a section and notifies the user when they try to change sections
function checkSave(targetSection){
    if(changedQuestions.isEmpty()){
	   sectionBuilder(targetSection);
    }else{
    	if($('.alert-warning').length != 0){
    	    sectionBuilder(targetSection);
    	}else{
    	    insertWarning("You have unsaved responses. Please click the Save Progress button at the bottom of the page or your changes will be lost.", "alert-warning");
        }
    }
}

//some questions are optional, this goes through the optional questions and
//sets up the proper dependencies on the front end to allow for instant activation
//and deactivation of questions.
function questionRequirement(formQuestion, input){
   
    //general queries to grab parent question
    var parentAnswer = $('#AID_'+formQuestion.requiredPredecessorAnswersID);
    var radioPair;
    
    //checkboxes have a listener on the one checkbox, but radios need a 
    //listener to turn on the question and a listener to turn off the question.
    //this requires us to query the yes/no radio buttons and attach a listener to each.
    if(parentAnswer.attr("type")=="checkbox"){
    	if(!(parentAnswer.attr('onclick')))
    	    parentAnswer.attr("onclick", "activateQuestion("+formQuestion.requiredPredecessorAnswersID+")");
    }else if(parentAnswer.attr("type")=="radio"){
    	radioPair = $('[name="'+($('#AID_'+formQuestion.requiredPredecessorAnswersID).attr("name"))+'"]');
    	if(!(radioPair.attr('onclick'))){
    	    for(var j=0; j<radioPair.length; j++){
                radioPair[j].setAttribute("onclick", "activateQuestion("+formQuestion.requiredPredecessorAnswersID+")");
    	    }
    	}else{
            var multipleDependency = true;
           
            for(var j=0; j<radioPair.length; j++){
                var lookupID = parseInt(radioPair[j].getAttribute("id"));
                //checking to see if radio buttons in the same question
                //have different dependencies. 
                if((dependency.containsKey(lookupID))  ){
                    if(lookupID != formQuestion.requiredPredecessorAnswersID){
                        
                        //building the consolodated string of IDs that need to be updated
                        var changeString, activateString;
                        activateString = "Function(";
                        for(var x=0; x<radioPair.length; x++){
                            var secondLookup = parseInt(radioPair[x].getAttribute("id"));
                            activateString += "activateQuestion(" + secondLookup + ")";
                        }
                        activateString += ")";
                        activateString = activateString.replace(")a", "),a");
                        
                        //updating the activation attribute
                        for(var x=0; x<radioPair.length; x++){
                            radioPair[x].setAttribute("onclick", activateString);
                        }
                    }
                }else{
                    multipleDependency = false;
                }
            }
        }
    }
    
    //We want to make sure that all children are disabled when their parents are disabled.
    if(parentAnswer.attr("disabled") == "disabled"){
        $('[name = "QID_'+formQuestion.question.questionsID+'"]').attr("disabled", true);
    }

    //Below we set up a data structure to hold parent/child relationships. This includes setting up
    //a hash map where they key is a question ID and the value is a set of question IDs that the key effects

    //checking to see if there is already an established key for this answer
    if(dependency.containsKey(formQuestion.requiredPredecessorAnswersID)){
	
        //checking to see if the parent answer is checked or not, which directly affects whether the child is active/deactive
        if( (!(parentAnswer.is(':checked'))) && (!(rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID))) ) {
            if((input.getAttribute("type") == "radio") ||(input.getAttribute("type") == "checkbox")){
                $('[name="QID_'+formQuestion.question.questionsID+'"]').attr("disabled", true);
            }else{ 
                input.setAttribute("disabled",true);
            }

            //points to an off page dependency. In this case we remove the question completely.
            if((!(rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID))) && ($('#AID_'+formQuestion.requiredPredecessorAnswersID).length == 0)){
                $('[name="QID_'+formQuestion.question.questionsID+'"]').parent().parent().hide();
            }

        }
    	
    	//adding id to the set of questions affected by an answer.
    	if(!(dependency.get(formQuestion.requiredPredecessorAnswersID)).contains(formQuestion.question.questionsID)){
    	    (dependency.get(formQuestion.requiredPredecessorAnswersID)).add(formQuestion.question.questionsID);
    	}
	
    }else{
    	//instantiating the answer in the dependency map and giving it a set of questions it affects. 
        //Going through the same logic as above.
    	var temp = new HashSet();
    	temp.add(formQuestion.question.questionsID);
    	dependency.put(formQuestion.requiredPredecessorAnswersID, temp);

        if( (!(parentAnswer.is(':checked'))) && (!(rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID))) ) {
            if((input.getAttribute("type") == "radio") || (input.getAttribute("type") == "checkbox")){
                $('[name="QID_'+formQuestion.question.questionsID+'"]').attr("disabled", true);
            }else{
                input.setAttribute("disabled",true);
            }

            if((!(rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID))) && ($('#AID_'+formQuestion.requiredPredecessorAnswersID).length == 0)){
                $('[name="QID_'+formQuestion.question.questionsID+'"]').parent().parent().hide();
            }
        }
    }
};

//This function toggles if a question is activated or
//deactivated.
function activateQuestion(answersId){
    var set = (dependency.get(answersId)).values();
    
    if(recentStage != "2"){
        for(var i=0; i<set.length; i++){
            if($('#AID_'+answersId).attr("disabled")){
                $('[name="QID_'+set[i]+'"]').attr("disabled", true);
            }else{
                if(!($('#AID_'+answersId).is(':checked'))){
        	       $('[name="QID_'+set[i]+'"]').attr("disabled", true);
        	   }else{
        	       $('[name="QID_'+set[i]+'"]').attr("disabled", false);
        	   } 
           }   
           //recursively going through the quetions to activate/deactivate children
           var setQuestions = $('[name="QID_'+set[i]+'"]');
            for(var j=0; j<setQuestions.length; j++){
                var setID = setQuestions[j].getAttribute("id");
                if(dependency.containsKey(parseInt(setID))){
                    activateQuestion(parseInt(setID));
                }
           }
        }
    }
};

//Builds the buttons that appear at the bottom of the page 
function progressBuilder(targetSection){
    var spButton;

    //Cancel Button. Takes the user back to the home page.
    csButton = document.createElement("button");
    csButton.setAttribute("value", "Cancel");
    csButton.setAttribute("type", "button");
    csButton.setAttribute("class", "btn btn-default pull-left");
    csButton.setAttribute("title", "Cancel");
    csButton.setAttribute("onclick", "location.href='../SitePages/Home.aspx'");

    var text = document.createTextNode("Cancel");
    csButton.appendChild(text);
    $('div.progress-container').append(csButton);

    //save progress button. Runs the function to package the user's responses.
    spButton = document.createElement("button");
    spButton.setAttribute("value", "Save Progress");
    spButton.setAttribute("type", "button");
    spButton.setAttribute("class", "btn btn-default pull-right");
    spButton.setAttribute("title", "Save Progress");
    spButton.setAttribute("onclick", "createResponses()");

    var text = document.createTextNode("Save Progress");
    
    spButton.appendChild(text);
    $('div.progress-container').append(spButton);
};

//Builds the buttons that appear on top of the sidebar which
//allow users to upload documents and print a section
function actionBarBuilder(targetSection){
    var actionTools, printList, printLink, printIcon;
    var uploadList, uploadLink, uploadIcon, uploadInput;
    var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";    

    //general structure for print button
    actionTools = document.createElement("ul");
    actionTools.setAttribute("class", "actionTools");

    printList = document.createElement("li");
    
    //href will print the current webpage
    printLink = document.createElement("a");
    printLink.setAttribute("href", "javascript:window.print()");
    printLink.setAttribute("title", "");

    printIcon = document.createElement("span");
    printIcon.setAttribute("class", "glyphicon glyphicon-print");
    printLink.appendChild(printIcon);

    var text = document.createTextNode("Print Section " + sectionTitle.charAt(targetSection));
    printLink.appendChild(text);

    printList.appendChild(printLink);
    actionTools.appendChild(printList);

    //general structure for upload button
    //creating the normal button that stays hidden
    uploadInput = document.createElement("input");
    uploadInput.setAttribute("id", "upload-general");
    uploadInput.setAttribute("type", "file");
    uploadInput.setAttribute("name", "0000");
    uploadInput.style.display = "none";
    actionTools.appendChild(uploadInput);

    uploadList = document.createElement("li");

    //setting up the stylized button that triggers the usual
    //file upload button which is hidden.
    uploadLink = document.createElement("a");
    uploadLink.setAttribute("href", "#");
    uploadLink.setAttribute("onclick", "Function($('#upload-general').trigger('click'), changedQuestions.add('0000'))");

    uploadLink.style.cursor = "pointer";
    
    uploadIcon = document.createElement("span");
    uploadIcon.setAttribute("class", "glyphicon glyphicon-upload");
    uploadLink.appendChild(uploadIcon);

    var text = document.createTextNode("Upload Document(s)");
    uploadLink.appendChild(text);

    uploadList.appendChild(uploadLink);
    actionTools.appendChild(uploadList);
    

    $('div.actionbar').append(actionTools);
};

//creates the rest of the sidebar including the sections
//and the buttons that allow you to review and submit
function wellSidebarBuilder(targetSection){
    var listGroup, lGroup;
        
    $('.list-group').remove();

    listGroup = document.createElement("div");
    listGroup.setAttribute("class", "list-group");

    //creating the sidebar menu button for each section
    for(var i=0; i<study.form.containedFormCollection.length; i++){
    	var link = document.createElement("a");
        link.style.cursor = "pointer";

        //setting up the activated section styling
    	if(i == targetSection){
    	    link.setAttribute("class", "list-group-item active");
    	}else{
    	    link.setAttribute("class", "list-group-item");
    	}
    	
    	var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    	var text = document.createTextNode("Section " + sectionTitle.charAt(i));
    	link.appendChild(text);
    	
    	listGroup.appendChild(link);
    }
    
    $('div.well.sidebar').append(listGroup);

    lGroup = document.createElement("div");
    lGroup.setAttribute("class", "list-group");

    //setting up the Attachments button
    var link = document.createElement("a");
    link.style.cursor = "pointer";
    link.setAttribute("class", "list-group-item");
    
    var text = document.createTextNode("Attachments");
    link.appendChild(text);
    
    var icon = document.createElement("span");
    icon.setAttribute("class", "badge");
    
    var text = document.createTextNode("0");
    icon.appendChild(text);

    link.appendChild(icon);

    lGroup.appendChild(link);

    //setting up the Review & Submit button
    var link = document.createElement("a");
    link.style.cursor = "pointer";
    link.setAttribute("class", "list-group-item");
    link.setAttribute("onclick", "reviewApp()");
    
    var text = document.createTextNode("Review & Submit");
    link.appendChild(text);
    
    var icon = document.createElement("span");
    icon.setAttribute("class", "glyphicon glyphicon-chevron-right");
    
    link.appendChild(icon);

    lGroup.appendChild(link);

    $('div.well.sidebar').append(lGroup);
    
    //updating sidebar icons
    updateSidebarIcons(targetSection);
};

//organizes questions in sections to be either answered or unanswered and then
//reports to the user which questions they still need to fill out.
function validateQuestions(){
    
    formsResponse = study;
    rollupResponse = rollup;
    
    var form_id_questions_id_map = {};
    var questions_predecessor_map = {};
    var formValidCont, sectionCont, questionCont, text, anchor;
    var formTable, row, questionTable;

    formTable = document.createElement("table");
    questionTable = document.createElement("table");

    //putting together a map of parent/child questions
    for (var x = 0; x < formsResponse.form.containedFormCollection.length;x++) {
    	for(var xx = 0; xx < formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection.length; xx++){
    	    if (formsResponse.form.containedFormCollection[x].containedForm.formsID in form_id_questions_id_map) {
    		  form_id_questions_id_map[formsResponse.form.containedFormCollection[x].containedForm.formsID].push(formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID)
    		}else form_id_questions_id_map[formsResponse.form.containedFormCollection[x].containedForm.formsID] = [ formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID   ]; 
    	    
    	    if (formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection[xx].requiredPredecessorAnswersID) {
    		questions_predecessor_map[formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID] = formsResponse.form.containedFormCollection[x].containedForm.formQuestionCollection[xx].requiredPredecessorAnswersID;
    	    }
    	}
    }

    var form_id_empty = [];
    var form_id_incomplete = [];
    
    var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    formValidCont = document.createElement("div");
    sectionCont = document.createElement("div");

    //grabbing questions that still need to be answered
    for(var x = 0; x<rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection.length; x++){
    		
    	if(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[x].status==="INCOMPLETE"){
    	    form_id_incomplete.push(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[x].formsID);
    	
    	}else if(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[x].status==="EMPTY"){
            form_id_empty.push(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[x].formsID);
            //notifying users of sections that are completely empty
    	    row = document.createElement("tr");
    	    row.appendChild(createCell("td", "Section " +sectionTitle.charAt(x)+ ": " + study.form.containedFormCollection[x].containedForm.title + " is empty", null));
    	    formTable.appendChild(row);
    	}
    }

    formValidCont.appendChild(formTable);
    
    //below we grab questions that have been answered
    var filled_questions = [];

    for(var x = 0; x<rollupResponse.questionResponseUpdates.responses.length; x++){
	   filled_questions.push(rollupResponse.questionResponseUpdates.responses[x].questionsID);
    }

    var filled_answers = [];

    for(var x = 0; x<rollupResponse.questionResponseUpdates.responses.length; x++){
    	if(rollupResponse.questionResponseUpdates.responses[x].answersID)
    	    filled_answers.push(rollupResponse.questionResponseUpdates.responses[x].answersID);
    }
    
    function hasPredecessorAnswered(question){
    	if (filled_answers.indexOf(questions_predecessor_map[question]) == -1){
    	    return false;
    	}else{
    	    return true;
        }
    };
    
    var incomplete_forms_questions_map = {};

    //putting together a map of incomplete sections as the key and unanswered questions and the values
    for(var x = 0; x < form_id_incomplete.length; x++){
    	for(var xx = 0; xx < form_id_questions_id_map[form_id_incomplete[x]].length; xx++){
    	    if(filled_questions.indexOf(form_id_questions_id_map[form_id_incomplete[x]][xx])==-1){
        		if(!( form_id_questions_id_map[form_id_incomplete[x]][xx] in questions_predecessor_map )){
        		    if ( form_id_incomplete[x] in incomplete_forms_questions_map ) {
        			 incomplete_forms_questions_map[form_id_incomplete[x]].push(form_id_questions_id_map[form_id_incomplete[x]][xx]);
        		    }else {
        			 incomplete_forms_questions_map[form_id_incomplete[x]] = [ form_id_questions_id_map[form_id_incomplete[x]][xx] ];
        		    }
        		}else {
        		    if(hasPredecessorAnswered(form_id_questions_id_map[form_id_incomplete[x]][xx])){
            			if ( form_id_incomplete[x] in incomplete_forms_questions_map ) {
            			    incomplete_forms_questions_map[form_id_incomplete[x]].push(form_id_questions_id_map[form_id_incomplete[x]][xx]);
            			}else {
        			         incomplete_forms_questions_map[form_id_incomplete[x]] = [ form_id_questions_id_map[form_id_incomplete[x]][xx] ];
                        }
        		    } 
        		}
    	    }
    	}
    }

    //going through the map that was just created to notify the users of what specific questions still need to be answered.
    for(var x=0; x<study.form.containedFormCollection.length; x++){
    	for(var i=0; i<form_id_incomplete.length; i++){
    	    if(form_id_incomplete[i] == study.form.containedFormCollection[x].containedForm.formsID){
        		for(var y=0; y<study.form.containedFormCollection[x].containedForm.formQuestionCollection.length; y++){
                    for(var key in incomplete_forms_questions_map){
            		    for(var j=0; j<incomplete_forms_questions_map[key].length; j++){
                			if(incomplete_forms_questions_map[key][j] == study.form.containedFormCollection[x].containedForm.formQuestionCollection[y].question.questionsID){
                                row = document.createElement("tr");
                			    row.appendChild(createCell("td", "In Section " +sectionTitle.charAt(x)+ ", you still need to answer: " + study.form.containedFormCollection[x].containedForm.formQuestionCollection[y].question.questionHyperText, null));
                			    questionTable.appendChild(row);
                            }
            		    }
                    }
        		}
    	    }
    	}
    }

    formValidCont.appendChild(questionTable);

    //revealing the buttons to generate the hipaa and consent form.
    if((rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[12].status!=="EMPTY") || (rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[13].status!=="EMPTY")){
        //putting together the hipaa form button
        if(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[12].status!=="EMPTY"){

            var generate = document.createElement("button");
            generate.setAttribute("onclick", "generateHipaa('15')");
            var text = document.createTextNode("Generate HIPAA");
            generate.appendChild(text);
            formValidCont.appendChild(generate);
        }

        //putting together the consent form button
        if(rollupResponse.questionResponseUpdates.formValidationReport.formStatusCollection[13].status!=="EMPTY"){

            var generate = document.createElement("button");
            generate.setAttribute("onclick", "generateHipaa('14')");
            var text = document.createTextNode("Generate ICF");
            generate.appendChild(text);
            formValidCont.appendChild(generate);
        }

    }else{
            //hinting to the users that they still need to complete the hipaa and consent form
            var breaker = document.createElement("br");
            formValidCont.appendChild(breaker);

            var paragraph = document.createElement("p");
            var text = document.createTextNode("When you have completed both the HIPAA and ICF sections, you may generate documents.");
            paragraph.appendChild(text);
            formValidCont.appendChild(paragraph);
    }

    //checking the completeness of the application. All of the sections have to be labled as complete
    var complete = true;
    var expedited_complete = true;
    for(var x=0; x<rollup.questionResponseUpdates.formValidationReport.formStatusCollection.length; x++){
        if(rollup.questionResponseUpdates.formValidationReport.formStatusCollection[x].status != "COMPLETE")
            complete = false;
    }

    //if a user is submitting an application as an expedited study, then they need to meet the three requirements, which the below logic looks for
    if(rollupAnswers.contains(study.form.containedFormCollection[0].containedForm.formQuestionCollection[4].question.answerCollection[1].answersID)){
        if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID)))
            expedited_complete = false;

        if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID)))
            expedited_complete = false;

        if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[2].answersID)))
            expedited_complete = false;
    }

    //finally, for a user to submit an application, they have to either be a PI or CO-PI
    if(complete){
        if(expedited_complete){
            if(studyPI.containsValue(currentUser.loginName)){
                //setting up the structure for a user to submit their application
                var submitCont = document.createElement("form");
                submitCont.setAttribute("action", "irb-applications.html");

                var breaker = document.createElement("br");
                formValidCont.appendChild(breaker);

                //inserting an official narrative for users to accept while submitting
                var paragraph = document.createElement("p");
                var text = document.createTextNode("I accept responsibility for assuring that this protocol will be conducted in compliance with ethical clinical practices and all applicable VA, IRB, FDA regulations, IRB and Institutional policies relative to the protection of the rights, welfare and safety of human subjects.  I acknowledge that I have the primary and ultimate responsibility for protecting the rights and welfare of research participants and that I understand the ethical principles of human subject protection.  I affirm that I have the competencies and adequate resources and facilities to carry out the proposed research and I attest that this application is scientifically and ethically sound.  Finally, all Conflicts of Interests have been disclosed.");
                paragraph.appendChild(text);
                submitCont.appendChild(paragraph);

                var paragraph = document.createElement("p");
                var text = document.createTextNode("In submitting this application, I certify the acknowledgements previously described and that the research team is qualified and credentialed to conduct this research study.  I and my study team will conduct and oversee this project in accordance with all ethical standards required by the VA for the conduct of human subjects research.");
                paragraph.appendChild(text);
                submitCont.appendChild(paragraph);

                var submission = document.createElement("button");
                submission.setAttribute("onclick", "submitIRB()");
                var text = document.createTextNode("Submit Application");
                submission.appendChild(text);
                submitCont.appendChild(submission);

                formValidCont.appendChild(submitCont);
            }else{
                var breaker = document.createElement("br");
                formValidCont.appendChild(breaker);

                //notifying users why their application is incomplete
                var paragraph = document.createElement("p");
                var text = document.createTextNode("You must be either the PI or CO-PI to submit an application");
                paragraph.appendChild(text);
                formValidCont.appendChild(paragraph);
            }
        }else{
            var breaker = document.createElement("br");
            formValidCont.appendChild(breaker);

            //notifying users why their application is incomplete
            var paragraph = document.createElement("p");
            var text = document.createTextNode("You have idenitfied your application as being expedited, however you haven't met all three expedited cagtegories outlined in Section C");
            paragraph.appendChild(text);
            formValidCont.appendChild(paragraph);
        }
    }else{
            var breaker = document.createElement("br");
            formValidCont.appendChild(breaker);

            //notifying users why their application is incomplete
            var paragraph = document.createElement("p");
            var text = document.createTextNode("You must complete the IRB application before you can submit.");
            paragraph.appendChild(text);
            formValidCont.appendChild(paragraph);
     }

    return formValidCont;
};

//changes the status of the application from "draft" to "review"
function submitIRB(){
    var desiredStage = recentStage + 1;
    var userID;

    for(var i=0; i<users.user.length; i++){
        if(users.user[i].displayName == currentUser.displayName)
            userID = users.user[i].usersID;
    }
    sendREST("POST", "manage/study/"+assignedID+"/stage/Review?user=" + userID, null, false);
};

//opens a new tab and starts the hipaa or icf download.
function generateHipaa(id){
    var xmlhttp;

    if(id == "13"){
        window.open("http://ram-spdev.aac.dva.domain/sites/RAMS_UAT/Documents/Form%20Templates/HIPAAForm.docx");
    }else if(id == "12"){
        window.open("http://ram-spdev.aac.dva.domain/sites/RAMS_UAT/Documents/Form%20Templates/ConsentForm.docx");
    }else if(id == "15"){
        window.open(restPath.concat("study/"+assignedID+"/documentGeneration/1/13"));
    }else if(id == "14"){
        window.open(restPath.concat("study/"+assignedID+"/documentGeneration/1/14"));
    }
};

//puts together the content of the popup modal that allows users to see what files
//they've uploaded, delete uploaded files, what questions they have left to answer, allows 
//users to generate the hipaa and consent form, and submit the application.
function reviewApp(){
    var reviewCont, uploaded, fileTable, tableCont, fileHead, row, fileBody;
    
    //removing any trace of a previous popup modal
    $('#review').remove();

    //putting together the basic structure and style of the popup 
    reviewCont = document.createElement("div");
    reviewCont.setAttribute("id", "review");
    reviewCont.setAttribute("class", "well");
    reviewCont.setAttribute("style", "background: #fff");

    //getting all of the files
    uploaded = sendREST("GET", "fascade/file/querystudy/" + assignedID, null, false);
    
    var text = document.createTextNode("Below are all the files that you have uploaded:");
    reviewCont.appendChild(text);
    
    tableCont = document.createElement("div");
    tableCont.setAttribute("class", "col-xs-12");

    fileTable = document.createElement("table");
    fileTable.setAttribute("class", "table table-bordered table-striped table-hover");

    fileHead = document.createElement("thead");
    
    row = document.createElement("tr");
    row.appendChild(createCell("th", "File", null));
    row.appendChild(createCell("th", "Remove", null));

    fileHead.appendChild(row);
    fileTable.appendChild(fileHead);

    fileBody = document.createElement("tbody");
    
    //putting together the table of files
    if((uploaded.files === undefined)||(uploaded.files.length == 0)){
    	row = document.createElement("tr");
    	row.appendChild((createCell("td", "No Files Uploaded", null)));
    	fileBody.appendChild(row);
    	fileTable.appendChild(fileBody);
    }else{
    	for(var i=0; i<uploaded.files.length; i++){
    	    row = document.createElement("tr");
    	    row.setAttribute("name", uploaded.files[i].file);
    	    row.appendChild(createCell("td", uploaded.files[i].file, null));
    	    row.appendChild(createCell("td", "delete", uploaded.files[i].file, uploaded.files[i].details[0].tags));
    	    fileBody.appendChild(row);
    	}
	   fileTable.appendChild(fileBody);
    }
    tableCont.appendChild(fileTable);
    reviewCont.appendChild(tableCont);
    
    var text = document.createTextNode("Review your application below:");
    reviewCont.appendChild(text);

    //validate questions inserts the rest of the reviewing content
    reviewCont.appendChild(validateQuestions());

    var breaker = document.createElement("br");
    reviewCont.appendChild(breaker);

    //button to close the window
    var closer = document.createElement("button");
    closer.setAttribute("type", "button");
    closer.setAttribute("onclick", "Function($('#review').popup('toggle'))");

    var text = document.createTextNode("Close Window");
    closer.appendChild(text);

    reviewCont.appendChild(closer);

    //specific library code to show popup
    $('body').append(reviewCont);
    $('#review').popup('show');
};

//creates a cell in a table. type is what kind of cell to create,
//content is what goes in the cell, tag is used to pass file
//information (since the only table is used with files)
function createCell(type, content, tag, id){
    
    var cell = document.createElement(type);
    
    var text = document.createTextNode(content);
    
    if(content == "delete"){
        //styling delete text
    	var anchor = document.createElement("a");
    	anchor.style.color = "blue";
    	anchor.style.textDecoration = "underline";
    	anchor.style.textDecorationColor = "blue";
    	anchor.style.cursor = "pointer";
    	
    	anchor.appendChild(text);
    	cell.appendChild(anchor);

        //tag here is used to link the remove button to a specific file
    	cell.setAttribute("onclick", "removeFile('"+tag+"', "+id+")");
    }else if(content == "No Files Uploaded"){
    	cell.appendChild(text);
    	cell.setAttribute("colspan", "2");
    }else{
        cell.appendChild(text);
    }
    
    return cell;
};

//rest call for removing a file from the cache and from
//the table
function removeFile(file, id){
    var status;
    var qResponse = {};
    var data = [];
    var response = {};

    status = sendREST("DELETE", "fascade/file/delete/study/"+assignedID+"/filename/"+file, null, false);
    $('[name="'+file+'"]').remove();

    response = {
        "questionsID": id
    };

    data.push(response);

    //properly packaging to send through REST call
    qResponse = {
    "questionResponse": data
    };

    sendREST("POST", "study/"+assignedID+"/responses/"+recentStage, qResponse, false);

};

//assigning the appropriate icons to the buttons in the sidebar
function updateSidebarIcons(targetSection){
    //grabbing all of the section buttons
    var sideButtons = $('.list-group-item');
    var incomplete=0;
    var complete=0;
    var empty=0;
    var total;
    
    //removing previous icons
    $('.glyphicon-adjust').remove();
    $('.glyphicon-record').remove();
    $('.glyphicon-ok-circle').remove();
    
    getRollup();

    //going through the most recent user's responses and assigning them the correct icons as
    //well as giving the relevant buttons a link.
    for(var i=0; i<sideButtons.length-2; i++){
    	if(assignedID == "1"){
            if(i == 0){
                var icon = document.createElement("span");
                icon.setAttribute("class", "glyphicon glyphicon-record");
                sideButtons[i].appendChild(icon);
            }

    	    sideButtons[i].setAttribute("onclick", "insertWarning('You must complete Section A and Save your responses before proceeding.', 'alert-warning')");
    	}else{
    	    var icon = document.createElement("span");
    	    if(rollup.questionResponseUpdates.formValidationReport.formStatusCollection[i].status == "INCOMPLETE"){
        		icon.setAttribute("class", "glyphicon glyphicon-adjust");
        		incomplete += 1;
        		sideButtons[i].setAttribute("onclick", "checkSave("+i+")");
                sideButtons[i].setAttribute("style", "cursor: pointer");
    	    }else if(rollup.questionResponseUpdates.formValidationReport.formStatusCollection[i].status == "NOT_APPLICABLE"){
                //when a section isn't required a user shouldn't be able to click on it and
                //it shouldn't have an icon
                sideButtons[i].removeAttribute("onclick");
                sideButtons[i].setAttribute("style", "cursor: not-allowed");
    	    }else if(rollup.questionResponseUpdates.formValidationReport.formStatusCollection[i].status == "EMPTY"){
                icon.setAttribute("class", "glyphicon glyphicon-record");
                sideButtons[i].setAttribute("onclick", "checkSave("+i+")");
                sideButtons[i].setAttribute("style", "cursor: pointer");
                empty += 1;
    	    }else if(rollup.questionResponseUpdates.formValidationReport.formStatusCollection[i].status == "COMPLETE"){
        		icon.setAttribute("class", "glyphicon glyphicon-ok-circle");
        		sideButtons[i].setAttribute("onclick", "checkSave("+i+")");
                sideButtons[i].setAttribute("style", "cursor: pointer");
        		complete += 1;
    	    }
    	    sideButtons[i].appendChild(icon);
    	}
    }
    //updates the general information section that gives the user
    //a quick glance of how much they have left to do
    total = incomplete+empty+complete;
    $('.sectionLocation').html("Completed: " + complete + "/" + total);
};

function createStudy(){
    var facilities, users, id, titleID, packaging;
    var userTable = new Hashtable();

    //grabs a new generated study id
    var temp = sendREST("GET", "fascade/studyidgenerator/sitecode/1111/humorani/H", null, false); 
    assignedID = temp.studyAssignedID;

    //grabs all the studies that have been created.
    studies = sendREST("GET", "manage/study", null, false);

    //formatting the recieved JSON
    if((studies.study.length === undefined) || (studies.study.length == 0)){
        studyID = 1;
    }else{
       var studyLength = studies.study.length - 1;
        studyID = studies.study[studyLength].studiesID + 1;
    }

    //grabbing all the users in the database and formatting the recieved JSON
    users = sendREST("GET", "user", null, false);
    if(users.user.length === undefined)
        users.user = [users.user];

    //checking to see if the user is in the sql database
    for(var i=0; i<users.user.length; i++){
        userTable.put(users.user[i].displayName, users.user[i].usersID);
    }

    //a new study has to be linked to a facility
    facilities = sendREST("GET", "facility", null, false);
    if(facilities.facility.length === undefined)
        facilities.facility = [facilities.facility];
           
    if(facilities.facility.length == 0){
        //inserting a facility if the database doesn't hold any
        
        var addFacility = {
            "administratorSPGroup": 0,
            "description": "initial facility",
            "facilitiesID": 0,
            "name": "General Site",
            "reviewerSPCalendar": "00000000-0000-0000-0000-000000000000",
            "reviewerSPGroup": 0,
            "safetySPGroup": 0
        };
        
        facility = {
            "facility": addFacility
        };
        
        sendREST("POST", "manage/facility", facility, false);
    }
    
    //packaging study
    facility = {
        "facilitiesID": facilities.facility[0].facilitiesID
    };

    //we know we are only creating the study when we are on the first page, grabbing the study title
    titleID = study.form.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;
    studyTitle = $("[name = 'QID_"+titleID+"']").val();

    packaging = {
        "assignedID": assignedID,
        "facility": facility,
        "studiesID": studyID,
        "title": studyTitle
    };

    var data = {
        "study": packaging
    };

    //putting the newly created study in the database
    sendREST("POST", "manage/study?user=" + userTable.get(currentUser.displayName), data, false);
};


function initialSectionCheck(){
    var section, required;
    var questionCollection = new HashSet();

    section = study.form.containedFormCollection[0].containedForm.formQuestionCollection; 

    for(var i=0; i<section.length; i++){
        required = true;

        //grabbing a specific question
        question = section[i].question.questionsID;

        if(section[i].requiredPredecessorAnswersID){
            //here we are looking into first level dependent questions. With the current 
            //section A, there isn't a question dependency where question C depends on question
            //B which depends on question A (second level dependency);
            var parent = section[i].requiredPredecessorAnswersID; 

            if(!($("#"+parent).is(':checked'))){
                required = false; 
            }
        }

        if(required){
            if((section[i].question.responseFormat.name == "radio") || (section[i].question.responseFormat.name == "checkbox")){
                var complete = false;
                var answers = $('[name="QID_'+question+'"]');

                for(var j=0; j<answers.length; j++){
                    var id = answers[j].getAttribute("id");

                    if($("#"+id).is(':checked')){
                        complete = true;
                    }
                }

                if(complete != true){
                    return false;
                }

            }else if(section[i].question.responseFormat.name == "select"){
                if($("[name = '"+question+"']").find(':selected').val() == "none"){
                    return false;
                }

            }else{
                if($("[name = '"+question+"']").val() == ""){
                    return false;
                }
            }
        }
    }
    return true;
}

//grabbing all user input and packaging it up to send to the database
function createResponses(){
    var changedPackage; 
    var qResponse = {};
    var data = [];
    var response = {};
    var question, uploaders; 

    if(assignedID == "1"){
        if(!initialSectionCheck()){
            insertWarning('You must complete Section A and save your responses before proceeding.', 'alert-warning');

            return;
        }else{

            createStudy();

            //Getting the most recent stage to use later
            studyResponse = sendREST("GET", "study/"+assignedID, null, false);
            if(!("length" in studyResponse.study.studyStatusCollection))
                studyResponse.study.studyStatusCollection = [studyResponse.study.studyStatusCollection];
            recentStage = studyResponse.study.studyStatusCollection.length;

            $('div.page-header').empty();
            headerBuilder(0);                        
        }
    }

    //looking for uploaded files to send to the cache
    uploaders = $('[type="file"]');
    for(var i=0; i<uploaders.length; i++){
    	if(uploaders[i].value){
    	    var file = uploaders[i].value.split("\\");

            //packaging up the file information and storing it in the cache
    	    var request = new XMLHttpRequest();
    	    request.open("POST", restPath.concat("fascade/file/upload/onlyfilename"), false);
            request.setRequestHeader("filename", file[file.length-1]);
            request.setRequestHeader("login_name", currentUser.displayName);
            request.setRequestHeader("study_id", assignedID);
            request.setRequestHeader("tags", (uploaders[i].name).replace('QID_', ''));
    	    request.send(null);

            response = {
                "questionsID": (uploaders[i].name).replace('QID_', ''),
                "response": file[file.length-1]
            };
            data.push(response);

    	    insertWarning('You have successfully saved your responses', 'alert-success');
    	    if(changedQuestions.contains(parseInt((uploaders[i].name).replace('QID_', '')))){
                changedQuestions.remove(parseInt((uploaders[i].name).replace('QID_', '')));
    	    }
    	}
    }

    if((studyTitleID != "0") && (changedQuestions.contains(studyTitleID))){

        studyTitle = $('[name="QID_'+studyTitleID+'"]').val();

        //packaging up the file information and storing it in the cache
        var request = new XMLHttpRequest();
        request.open("POST", restPath.concat("study/"+assignedID+"/newTitle"), true);
        request.setRequestHeader("newTitle", studyTitle);
        request.send(null);

        $('div.page-header').empty();
        headerBuilder(0);
    }

    //changed questions keeps track of what questions the user
    //has modified while filling out the current section. we only
    //want to package up and send these questions
    changedPackage = changedQuestions.values();
    changedQuestions = new HashSet();
    
    //Filtering through each changed question and packaging it up based on what
    //type of input it is.
    for(var i=0; i<changedPackage.length; i++){
        var toPackage = $('[name="QID_'+changedPackage[i]+'"], #AID_'+changedPackage[i]+'');
        for(var j=0; j<toPackage.length; j++){
            if((toPackage[j].type == "radio")) {
                    //have to grab the set of radios assigned to a questions and see 
                    //which one is actually selected to send to the database
                if(toPackage[j].checked == true){
                    response = {
                    "answersID": (toPackage[j].id).replace('AID_', ''),
                    "questionsID": (toPackage[j].name).replace('QID_', '')
                    };
                    data.push(response);
                }
            }else if((toPackage[j].type == "checkbox")){
                //there's a difference in how checked checkboxes
                //are sent vs how unchecked checkboxes are sent
                if(toPackage[j].checked == true){
                    response = {
                        "answersID": (toPackage[j].id).replace('AID_', ''),
                        "checkboxIndex": toPackage[j].title,
                        "questionsID": (toPackage[j].name).replace('QID_', '')
                    };
                data.push(response);
                }else if(toPackage[j].checked == false){
                    response = {
                        "checkboxIndex": toPackage[j].title,
                        "questionsID": (toPackage[j].name).replace('QID_', '')
                    };
                data.push(response);
                }
            }else if((toPackage[j].type == "text") || (toPackage[j].type == "textarea") || (toPackage[j].type == "select-one")){
                    //everything else is pretty straightforward to send
                    response = {
                    "questionsID": (toPackage[j].name).replace('QID_', ''),
                    "response": toPackage[j].value
                    };
                data.push(response);
            }
        } 
    }
    

    //properly packaging to send through REST call
    qResponse = {
	"questionResponse": data
    };


    //only making a xmlhttp request if user has updated questions
    if(data[0] != null)
        sendREST("POST", "study/"+assignedID+"/responses/"+recentStage, qResponse, false);

    rollupAnswers = new HashSet();

    //getting current target section to pass 
    if(data[0] != null){
        var sidebar = $('.list-group-item');
        var current = $('.list-group-item.active');
        for(var i=0; i<sidebar.length-2; i++){
            if(sidebar.get(i) == current.get(0)){
                //updating sidebar icons now that we have the section that
                //should show up as activated.
                updateSidebarIcons(i);

                //alerting users of exempt question constraints
                if(i == 2){
                    //if a user is submitting an application as an expedited study, then they need to meet the three requirements, which the below logic looks for
                    if(rollupAnswers.contains(study.form.containedFormCollection[0].containedForm.formQuestionCollection[4].question.answerCollection[1].answersID)){
                        if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID))){
                            insertWarning("You have selected that your application is expedited, but you haven't checked all three expedited categories", "alert-warning");
                        }else if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID))){
                            insertWarning("You have selected that your application is expedited, but you haven't checked all three expedited categories", "alert-warning");
                        }else if(!(rollupAnswers.contains(study.form.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[2].answersID))){
                            insertWarning("You have selected that your application is expedited, but you haven't checked all three expedited categories", "alert-warning");
                        }
                    }
                }
            }
        }
    }
};
