/***************************************************
E1.1.0 RAM Increment 1 Patch Code
January 2015
Department of Veteran's Affairs/Booz Allen Hamilton
dynamicDashboard.js
***************************************************/

//Linking in the sql database
var restPath = "http://vaphiramapp60.domain:8080/RAMS-0.0.1/";
//Linking in the sharepoint site
var siteUrl = "http://vaww.SQA.RAM.msp.domain/sites/IRBApplications/";

//template for username that is retrieved from sharepoint 
var retrievedUser = "testk";

//globals that have to be shared between functions
var facility, id;
var studyID = 1;

//Initial function called by HTML. kicks off the process
//to grab the username from sharepoint
function process(){
    //loading connection to sharepoint
    var clientContext = new SP.ClientContext(siteUrl);

    this.website = clientContext.get_web();
    this.currentUser_SP = website.get_currentUser();
    clientContext.load(currentUser_SP);
    //trying to grab username information from sharepoint
    clientContext.executeQueryAsync(Function.createDelegate(this, this.userQuerySucceeded), Function.createDelegate(this, this.userQueryFailed));
};

//When we know that we have successfully retrieved a username from
//shareponit, then we can set up the baseline information that is needed.
function userQuerySucceeded(sender, args) {
    var username_SP, split;
    
    //parsing out the needed username from sharepoint
    username_SP = currentUser_SP.get_loginName();
    split = username_SP.split("\\");
    retrievedUser = split[1];

    id = 1;

    //puts together the dashboard table
    createTable();
    checkUser();

    //removes HTML traces that allow user to leave dashboard page
    $('.start').attr("href", "irb-applications-general.html?" + id + "?" + retrievedUser);
	$('.start').html("Start IRB Application");
};

//notifying the user that their sharepoint username couldn't be retrieved
function userQueryFailed(sender, args) {
    alert('request failed ' + args.get_message() + '\n' + args.get_stackTrace());
};

//checks to see if the current user is in the sql database
function checkUser(){
    var users;
    var userTable = new HashSet();

    //grabbing all the users in the database and formatting the recieved JSON
    users = sendREST("GET", "user", null, false);
    if(users.user.length === undefined)
        users.user = [users.user];

    //checking to see if the user is in the sql database
    for(var i=0; i<users.user.length; i++){
        userTable.add(users.user[i].displayName);
    }

    if(!userTable.contains(retrievedUser)){
        var addUser = {
            "displayName": retrievedUser,
            "loginName": currentUser_SP.get_title(),
            "permanentLoginIdentifier": retrievedUser,
            "usersID": currentUser_SP.get_id()
        };

        var data = {
            "user": addUser
        };

        sendREST("POST", "manage/user", data, false);
    }
};

//puts together the dashboard table. This will list out any studies that
//the user is associated with and provide information on those studies.
function createTable(){
    var recentStage, row, studies;
    var noStudies = true;
    
    //grabs all the studies that have been created.
    studies = sendREST("GET", "manage/study", null, false);

    //formatting the recieved JSON
    if((studies.study.length === undefined) || (studies.study.length == 0)){
        studyID = 1;
    }else{
	   var studyLength = studies.study.length - 1;
        studyID = studies.study[studyLength].studiesID + 1;
    }

    //filtering through the studies and putting the necessary ones in the dashboard
    for(var i=0; i<studies.study.length; i++){
    	//accounting for recieved JSON
        if(studies.study[i].studyStatusCollection === undefined){
    	    continue;
    	}
    	
        //formatting the recieved JSON
    	if(studies.study[i].studyStatusCollection.length === undefined){
    	    studies.study[i].studyStatusCollection = [studies.study[i].studyStatusCollection];
        }

        //recentStage relates to whether or not the study is in draft or review
    	recentStage = studies.study[i].studyStatusCollection.length-1;
    		    
    	row = document.createElement("tr");
        if(studies.study[i].studyStatusCollection[recentStage].createdBy.displayName == retrievedUser){

    	    var cell = document.createElement("td");
    	    
            //allows the user to return to a previous study by clicking on this study ID link in the dashboard
    	    var anchor = document.createElement("a");
    	    anchor.setAttribute("href", "irb-applications-general.html?" + studies.study[i].assignedID + "?" + retrievedUser);
    	    anchor.setAttribute("title", studies.study[i].title);
    	    
            //study ID
    	    var text = document.createTextNode(studies.study[i].assignedID);
    	    anchor.appendChild(text);
    	    cell.appendChild(anchor);
    	    row.appendChild(cell);
    	    
    	    //study title
    	    row.appendChild(createCell(studies.study[i].title));
    	    
    	    //Type 
    	    row.appendChild(createCell("Type TBD"));
    	    
    	    //PI Name
    	    row.appendChild(createCell(studies.study[i].studyStatusCollection[recentStage].createdBy.displayName));
    	    
    	    //Status (draft or review)
    	    row.appendChild(createCell(studies.study[i].studyStatusCollection[recentStage].studyState.name));
    	    
    	    //Expiration Date
    	    row.appendChild(createCell("Expiration TBD"));
    	    
    	    //IRB Agenda Date
    	    row.appendChild(createCell("IRB DATE TBD"));
    	
    	    $('tbody').append(row);
    	    noStudies = false;
    	}	
    }

    if(noStudies){
    	//no studies linked to user
        row = document.createElement("tr");
    	var cell = document.createElement("td")
        var text = document.createTextNode("You have no applications");
        cell.appendChild(text);
        cell.setAttribute("colspan", 7);
    	cell.setAttribute("style", "text-align:center");
    	row.appendChild(cell);	    
        
    	$('tbody').append(row);
    }
};

//creates a cell in a table. takes in what should be shown
//in the cell
function createCell(displayText){
    
    var cell = document.createElement("td");
    
    var text = document.createTextNode(displayText);
    cell.appendChild(text);
    
    return cell;
};

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, sync){
    var xmlResponse;

    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
        xmlResponse = new XMLHttpRequest();
    }else{
        xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
        xmlResponse.open(httpMethod, restPath.concat(endpoint), sync);
        xmlResponse.setRequestHeader("Content-Type", "application/json");

        if((httpMethod == "PUT")||(httpMethod == "DELETE")){
            xmlResponse.setRequestHeader("sm_user", data);
            xmlResponse.send(null);

            //204 is the response status indicating success, while
            //403 indicates failure
            if(xmlResponse.status === 204){
                return true;
            }else if(xmlResponse.status === 403){
                return false;
            }
        }
        if(httpMethod == "POST"){
            try{
                //sending data
                xmlResponse.send(JSON.stringify(data));
            }catch(e){
                alert(e.toString());
            }
        }
        if(httpMethod == "GET"){
            //retrieving the needed data
            xmlResponse.onreadystatechange = function(){
                if(xmlResponse.readyState === 4){
                    if(xmlResponse.status === 200){
                        try{
                            xmlResponse = JSON.parse(xmlResponse.responseText);

                        }catch(e){
                            alert(e.toString());
                        }

                    }else{
                        alert(xmlResponse.statusText);
                    }
                }
            };
            xmlResponse.send(null);
            //passing the needed data to the function that is requesting it
            return xmlResponse;
        }

    }catch(e){
        alert(e.toString());
    }
};
