/***************************************************
E1.1.0 RAM Increment 1 Patch Code
January 2015
Department of Veteran's Affairs/Booz Allen Hamilton
dynamicDashboard.js
***************************************************/

//Linking in the sql database
var restPath = "http://vaphiramapp60.domain:8080/RAMS-0.0.1/";
//Linking in the sharepoint site
var siteUrl = "http://vaww.SQA.RAM.msp.domain/sites/IRBApplications/";

//globals that have to be shared between functions
var retrievedUser, id;
    
function process(){
    //loading connection to sharepoint
    var clientContext = new SP.ClientContext(siteUrl);
    
    this.website = clientContext.get_web();
    this.currentUser_SP = website.get_currentUser();
    clientContext.load(currentUser_SP);
    //trying to grab username information from sharepoint
    clientContext.executeQueryAsync(Function.createDelegate(this, this.userQuerySucceeded), Function.createDelegate(this, this.userQueryFailed));
};

//When we know that we have successfully retrieved a username from
//shareponit, then we can set up the baseline information that is needed.
function userQuerySucceeded(sender, args) {
    var username_SP, split;
    
    //parsing out the needed username from sharepoint
    username_SP = currentUser_SP.get_loginName();
    split = username_SP.split("\\");
    retrievedUser = split[1];
        
    id = 1;

    checkUser();

    $('.start').attr("href", "irb-applications-general.html?" + id + "?" + retrievedUser);
	$('.start').html("Start IRB Application");
};

function userQueryFailed(sender, args) {
    alert('request failed ' + args.get_message() + '\n' + args.get_stackTrace());
};

//checks to see if the current user is in the sql database
function checkUser(){
    var users;
    var userTable = new HashSet();

    //grabbing all the users in the database and formatting the recieved JSON
    users = sendREST("GET", "user", null, false);
    if(users.user.length === undefined)
        users.user = [users.user];

    //checking to see if the user is in the sql database
    for(var i=0; i<users.user.length; i++){
        userTable.add(users.user[i].displayName);
    }

    if(!userTable.contains(retrievedUser)){
        var addUser = {
            "displayName": retrievedUser,
            "loginName": currentUser_SP.get_title(),
            "permanentLoginIdentifier": retrievedUser,
            "usersID": currentUser_SP.get_id()
        };

        var data = {
            "user": addUser
        };

        sendREST("POST", "manage/user", data, false);
    }
};

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, sync){
    var xmlResponse;

    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
        xmlResponse = new XMLHttpRequest();
    }else{
        xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
        xmlResponse.open(httpMethod, restPath.concat(endpoint), sync);
        xmlResponse.setRequestHeader("Content-Type", "application/json");

        if((httpMethod == "PUT")||(httpMethod == "DELETE")){
            xmlResponse.setRequestHeader("sm_user", data);
            xmlResponse.send(null);

            //204 is the response status indicating success, while
            //403 indicates failure
            if(xmlResponse.status === 204){
                return true;
            }else if(xmlResponse.status === 403){
                return false;
            }
        }
        if(httpMethod == "POST"){
            try{
                //sending data
                xmlResponse.send(JSON.stringify(data));
            }catch(e){
                alert(e.toString());
            }
        }
        if(httpMethod == "GET"){
            //retrieving the needed data
            xmlResponse.onreadystatechange = function(){
                if(xmlResponse.readyState === 4){
                    if(xmlResponse.status === 200){
                        try{
                            xmlResponse = JSON.parse(xmlResponse.responseText);

                        }catch(e){
                            alert(e.toString());
                        }

                    }else{
                        alert(xmlResponse.statusText);
                    }
                }
            };
            xmlResponse.send(null);
            //passing the needed data to the function that is requesting it
            return xmlResponse;
        }

    }catch(e){
        alert(e.toString());
    }
};
