/*
* E1.0.0 RAM Increment 1 Final Code
* December 2014
* Department of Veterans Affairs/Booz Allen Hamilton 
* 
*/

package gov.va.rams.entities;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

/**
 *
 * @author Brian
 */
@Entity
@Table(name = "Studies", uniqueConstraints = {
    @UniqueConstraint(columnNames = {"AssignedID"})})
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Study.findByAssignedID", query = "SELECT s FROM Study s WHERE s.assignedID = :assignedID"),
    @NamedQuery(name = "Study.findAll", query = "SELECT s FROM Study s")})
public class Study implements Serializable {
    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @NotNull
    @Column(nullable = false, updatable = false)
    private Integer studiesID;
    @Basic(optional = false)
    @NotNull
    @Size(min = 1, max = 15)
    @Column(nullable = false, length = 15)
    private String assignedID;
    @Basic(optional = false)
    @NotNull
    @Size(min = 1, max = 200)
    @Column(nullable = false, length = 200)
    private String title;
    @Basic(optional = false)
    @NotNull
    @Column(nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    private Date synchronizedTimestamp;
    @JoinColumn(name = "FacilitiesID", referencedColumnName = "FacilitiesID", nullable = false)
    @ManyToOne(optional = false)
    private Facility facility;
    @JoinColumn(name = "ParentStudiesID", referencedColumnName = "StudiesID")
    @ManyToOne
    private Study parentStudy;
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER, mappedBy = "study")
    @OrderBy("stageCounter")
    private Collection<StudyStatus> studyStatusCollection;
    @Basic(optional = false)
    @NotNull
    @Column(nullable = false)
    private Integer formVersionsID;

    public Study() {
    }

    public Integer getStudiesID() {
        return studiesID;
    }

    public void setStudiesID(Integer studiesID) {
        this.studiesID = studiesID;
    }

    public String getAssignedID() {
        return assignedID;
    }

    public void setAssignedID(String assignedID) {
        this.assignedID = assignedID;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Date getSynchronizedTimestamp() {
        return synchronizedTimestamp;
    }

    public void setSynchronizedTimestamp(Date synchronizedTimestamp) {
        this.synchronizedTimestamp = synchronizedTimestamp;
    }

    public Facility getFacility() {
        return facility;
    }

    public void setFacility(Facility facility) {
        this.facility = facility;
    }

    @XmlTransient
    public Study getParentStudy() {
        return parentStudy;
    }

    public void setParentStudy(Study parentStudy) {
        this.parentStudy = parentStudy;
    }

    public Collection<StudyStatus> getStudyStatusCollection() {
        return studyStatusCollection;
    }

    public void setStudyStatusCollection(Collection<StudyStatus> studyStatusCollection) {
        this.studyStatusCollection = studyStatusCollection;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (studiesID != null ? studiesID.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Study)) {
            return false;
        }
        Study other = (Study) object;
        if ((this.studiesID == null && other.studiesID != null) || (this.studiesID != null && !this.studiesID.equals(other.studiesID))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "gov.va.rams.Study[ studiesID=" + studiesID + " ]";
    }

    public Integer getFormVersionsID() {
        return formVersionsID;
    }

    public void setFormVersionsID(Integer formVersionsID) {
        this.formVersionsID = formVersionsID;
    }
    
}
