/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package gov.va.rams.service;

import gov.va.rams.entities.Answer;
import gov.va.rams.entities.ContainedForm;
import gov.va.rams.entities.Facility;
import gov.va.rams.entities.Form;
import gov.va.rams.entities.FormQuestion;
import gov.va.rams.entities.FormVersion;
import gov.va.rams.entities.Question;
import gov.va.rams.entities.QuestionResponse;
import gov.va.rams.entities.ResponseFormat;
import gov.va.rams.entities.Study;
import gov.va.rams.entities.StudyState;
import gov.va.rams.entities.StudyStatus;
import gov.va.rams.entities.User;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;

/**
 *
 * @author Brian
 */
@Stateless
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
@Path("manage")
public class ManageServiceREST {
    @PersistenceContext(unitName = "RAMSPU")
    private EntityManager em;
    
    static private class ConditionalQuestion {
        FormQuestion formQuestion;
        Answer answer;
    }
    static private class ConditionalForm {
        ContainedForm form;
        Answer answer;
    }

    @PUT
    @Path("forms")
    @Consumes(MediaType.TEXT_PLAIN)
    public FormVersion addFormsPut(InputStream csv) throws IOException {
        return addForms(csv);
    }

    @POST
    @Path("forms")
    @Consumes(MediaType.TEXT_PLAIN)
    public FormVersion addForms(InputStream csv) throws IOException {
        FormVersion version = new FormVersion();
        HashMap<String, ResponseFormat> responseFormats = new HashMap<>();
        for (ResponseFormat rf : em.createNamedQuery("ResponseFormat.findAll", ResponseFormat.class).getResultList())
            responseFormats.put(rf.getName(), rf);
        HashMap<String, Answer> answersByName = new HashMap<>();
        LinkedList<ConditionalQuestion> conditionalQuestions = new LinkedList<>();
        LinkedList<ConditionalForm> conditionalForms = new LinkedList<>();

        Form root = new Form();
        root.setContainedFormCollection(new ArrayList<ContainedForm>());
        root.setIsPage(true);
        root.setNarrative("Go nuts!");
        root.setTitle("RAMS Root Form");
        version.setRootForm(root);
        
        CSVParser parser = new CSVParser(new InputStreamReader(csv), CSVFormat.EXCEL.withHeader());
        Form currentForm = null;
        Question currentQuestion = null;
        int currentFormPosition = 1;
        for (CSVRecord r : parser) {
            if (!r.get("Form Name").isEmpty()) {
                currentQuestion = null;
                currentFormPosition = 1;
                currentForm = new Form();
                currentForm.setIsPage(true);
                currentForm.setNarrative(r.get("Question Entry"));
                if (currentForm.getNarrative().isEmpty())
                    currentForm.setNarrative(r.get("Form Section"));
                currentForm.setTitle(r.get("Form Name"));
                currentForm.setFormQuestionCollection(new ArrayList<FormQuestion>());
                ContainedForm contain = new ContainedForm();
                contain.setContainerForm(root);
                contain.setContainedForm(currentForm);
                contain.setFormSequencePosition(1 + root.getContainedFormCollection().size());
                root.getContainedFormCollection().add(contain);
                final String dep = r.get("dependency1"); // dependency specification form 1
                if (!dep.isEmpty()) {
                    ConditionalForm cf = new ConditionalForm();
                    cf.form = contain;
                    cf.answer = answersByName.get(dep);
                    if (cf.answer != null)
                        conditionalForms.add(cf);
                }

            } else if (!r.get("Question Entry").isEmpty() || !r.get("Third level").isEmpty()) {
                currentQuestion = new Question();
                String text = r.get("Question Entry");
                if (text.isEmpty())
                    text = r.get("Third level");
                currentQuestion.setQuestionHyperText(text);
                String uiType = r.get("Type in UI");
                if (uiType.isEmpty())
                    uiType = "unknown";
                if (responseFormats.get(uiType) == null) {
                    ResponseFormat rf = new ResponseFormat();
                    rf.setName(uiType);
                    em.persist(rf);
                    responseFormats.put(uiType, rf);
                }
                currentQuestion.setResponseFormat(responseFormats.get(uiType));
                String help = r.get("Inline Help Message 1");
                if (!help.isEmpty())
                    currentQuestion.setHelpHyperText(help);
                currentQuestion.setAnswerCollection(new ArrayList<Answer>());
                FormQuestion contain = new FormQuestion();
                contain.setForm(currentForm);
                contain.setQuestion(currentQuestion);
                contain.setFormSequencePosition(currentFormPosition++);
                currentForm.getFormQuestionCollection().add(contain);
                final String dep = r.get("dependency1"); // dependency specification form 1
                if (!dep.isEmpty()) {
                    ConditionalQuestion cq = new ConditionalQuestion();
                    cq.formQuestion = contain;
                    cq.answer = answersByName.get(dep);
                    if (cq.answer != null)
                        conditionalQuestions.add(cq);
                }
            } else if (!r.get("Second Level").isEmpty() || !r.get("Fourth level").isEmpty()) {
                // Update the question with the answer type if it failed to get set on the question row.
                String uiType = r.get("Type in UI");
                if (!uiType.isEmpty()) {
                    if (responseFormats.get(uiType) == null) {
                        ResponseFormat rf = new ResponseFormat();
                        rf.setName(uiType);
                        em.persist(rf);
                        responseFormats.put(uiType, rf);
                    }
                    currentQuestion.setResponseFormat(responseFormats.get(uiType));
                }
                Answer answer = new Answer();
                answer.setQuestion(currentQuestion);
                String text = r.get("Second Level");
                if (text.isEmpty())
                    text = r.get("Fourth level");
                answer.setValue(text);
                currentQuestion.getAnswerCollection().add(answer);
                answer.setSequencePosition(currentQuestion.getAnswerCollection().size());
                final String qid = r.get("questionID");
                if (!qid.isEmpty())
                    answersByName.put(r.get("questionID"), answer);
            }
        }
        
        version.setStartedUsingAt(new Date());
        em.persist(version);
        em.flush();
        em.refresh(version);
        
        for (ConditionalQuestion dependency : conditionalQuestions)
            dependency.formQuestion.setRequiredPredecessorAnswersID(dependency.answer.getAnswersID());
        for (ConditionalForm dependency : conditionalForms)
            dependency.form.setRequiredPredecessorAnswersID(dependency.answer.getAnswersID());
        
        return version;
    }
    
    @POST
    @Path("user")
    public void addUser(User user) {
        user.setSynchronizedTimestamp(new Date());
        if (user.getDisplayName() == null)
            user.setDisplayName(user.getUsersID().toString());
        if (user.getLoginName() == null)
            user.setLoginName(user.getUsersID().toString());
        if (user.getPermanentLoginIdentifier() == null)
            user.setPermanentLoginIdentifier(user.getUsersID().toString());
        em.persist(user);
    }

    @POST
    @Path("facility")
    public void addFacility(Facility facility) {
        facility.setSynchronizedTimestamp(new Date());
        if (facility.getName() == null)
            facility.setName(facility.getFacilitiesID().toString());
        if (facility.getDescription() == null)
            facility.setDescription(facility.getName());
        if (facility.getReviewerSPCalendar() == null)
            facility.setReviewerSPCalendar("00000000-0000-0000-0000-000000000000");
        em.persist(facility);
    }

    @GET
    @Path("study")
    public List<Study> getStudies() {
        return em.createNamedQuery("Study.findAll", Study.class).getResultList();
    }

    @POST
    @Path("study")
    public void addStudy(Study study, @QueryParam("user") Integer usersID) {
        final Date timestamp = new Date();
        final User user = usersID != null ? em.find(User.class, usersID) : em.createNamedQuery("User.findAll", User.class).getResultList().get(0);
        StudyStatus ss = new StudyStatus();
        ss.setCreatedAt(timestamp);
        ss.setCreatedBy(user);
        ss.setUpdatedAt(timestamp);
        ss.setUpdatedBy(user);
        ss.setStageCounter(1);
        ss.setStudyState(getStudyState("draft"));
        ss.setStudy(study);
        study.setSynchronizedTimestamp(timestamp);
        if (study.getFormVersionsID() == null)
            study.setFormVersionsID(em.createNamedQuery("FormVersion.findNewest", FormVersion.class).getResultList().get(0).getFormVersionsID());
        study.setFacility(em.getReference(Facility.class, study.getFacility().getFacilitiesID()));
        study.setStudyStatusCollection(Collections.singletonList(ss));
        em.persist(study);
    }
    
    @POST
    @Consumes
    @Path("study/{id}/stage/{state}")
    public Study addStage(@PathParam("id") String id, @PathParam("state") String state, @QueryParam("user") Integer usersID) {
        final Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        final Date timestamp = new Date();
        final User user = usersID != null ? em.find(User.class, usersID) : em.createNamedQuery("User.findAll", User.class).getResultList().get(0);
        final StudyStatus ss = new StudyStatus();
        ss.setStudy(s);
        s.getStudyStatusCollection().add(ss);
        ss.setStageCounter(s.getStudyStatusCollection().size());
        ss.setStudyState(getStudyState(state));
        ss.setCreatedAt(timestamp);
        ss.setUpdatedAt(timestamp);
        ss.setCreatedBy(user);
        ss.setUpdatedBy(user);
        s.setSynchronizedTimestamp(timestamp);
        em.flush();
        em.refresh(s);
        return s;
    }
    
    private StudyState getStudyState(String name) {
        List<StudyState> results = em.createNamedQuery("StudyState.findByName", StudyState.class).setParameter("name", name).getResultList();
        if (results.isEmpty()) {
            StudyState ss = new StudyState();
            ss.setName(name);
            em.persist(ss);
            return ss;
        } else {
            return results.get(0);
        }
    }
}
