/*
* E1.0.0 RAM Increment 1 Final Code
* December 2014
* Department of Veterans Affairs/Booz Allen Hamilton 
* 
*/

package gov.va.rams.service;

import gov.va.rams.docx4j.DocXFormGenerator;
import gov.va.rams.entities.ContainedForm;
import gov.va.rams.entities.Facility;
import gov.va.rams.entities.Form;
import gov.va.rams.entities.FormQuestion;
import gov.va.rams.entities.FormVersion;
import gov.va.rams.entities.QuestionResponse;
import gov.va.rams.entities.Study;
import gov.va.rams.entities.StudyStatus;
import gov.va.rams.entities.User;
import gov.va.rams.service.elements.FormValidationReport;
import gov.va.rams.service.elements.QuestionResponseUpdates;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.EJB;
import javax.servlet.ServletContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.DELETE;
import javax.ws.rs.HeaderParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;

/**
 * 
 * @author Brian Feldman
 */

@Stateless
@Consumes(MediaType.APPLICATION_JSON)
@Produces(MediaType.APPLICATION_JSON)
@Path("")
public class ServiceREST {
    
    @javax.ws.rs.core.Context HttpHeaders requestHeaders;
    @javax.ws.rs.core.Context ServletContext context;

    @PersistenceContext(unitName = "RAMSPU")
    private EntityManager em;

    @GET
    @Path("forms")
    public List<FormVersion> getRootForms() {
        return em.createNamedQuery("FormVersion.findAll", FormVersion.class).getResultList();
    }
    
    @GET
    @Path("forms/{version}")
    public Form getRootForm(@PathParam("version") Integer version) {
        return em.find(FormVersion.class, version).getRootForm();
    }
    
    @GET
    @Path("user")
    public List<User> getUsers() {
        return em.createNamedQuery("User.findAll", User.class).getResultList();
    }

    @GET
    @Path("facility")
    public List<Facility> getFacilities() {
        return em.createNamedQuery("Facility.findAll", Facility.class).getResultList();
    }
    
    @GET
    @Path("study/{id}")
    public Study retrieveStudy(@PathParam("id") String id) {
        return em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
    }
    
    @GET
    @Path("study/{id}/responses/{stage}")
    public Collection<QuestionResponse> retrieveStudyResponsesByStage(@PathParam("id") String id, @PathParam("stage") Integer stage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        for (StudyStatus ss : s.getStudyStatusCollection())
            if (ss.getStageCounter() == stage)
                return ss.getQuestionResponseCollection();
        throw ServiceException.notFound();
    }

    @EJB
    private ResponseValidator responseValidator;
    
    @GET
    @Path("study/{id}/responses/{stage}/rollup")
    public QuestionResponseUpdates retrieveStudyResponsesRollup(@PathParam("id") String id, @PathParam("stage") Integer lastStage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        List<QuestionResponse> responsesByStage = new ArrayList<>();
        QuestionResponseUpdates updates = new QuestionResponseUpdates();
        StudyStatus status = null;
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            responsesByStage.addAll(ss.getQuestionResponseCollection());
            if (ss.getStageCounter() == lastStage) {
                updates.setUpdateTimestamp(ss.getUpdatedAt());
                status = ss;
                break;
            }
        }
        if (status == null)
            throw ServiceException.notFound();
        HashMap<QuestionResponse.Key, QuestionResponse> rollup = QuestionResponse.rollup(responsesByStage);
        updates.setResponses(rollup.values());
        updates.setFormValidationReport(responseValidator.getFormValidationReport(status));
        if (updates.getFormValidationReport() == null)
            updates.setFormValidationReport(responseValidator.updateFormValidationReport(status, rollup));
        return updates;
    }
    
    @GET
    @Path("study/{id}/responses/{stage}/validation")
    public FormValidationReport performStudyFormValidation(@PathParam("id") String id, @PathParam("stage") Integer lastStage) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        FormValidationReport report = null;
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            if (ss.getStageCounter() == lastStage) {
                report = responseValidator.getFormValidationReport(ss);
                break;
            }
        }
        if (report == null) {
            List<QuestionResponse> responsesByStage = new ArrayList<>();
            StudyStatus status = null;
            for (StudyStatus ss : s.getStudyStatusCollection()) {
                responsesByStage.addAll(ss.getQuestionResponseCollection());
                if (ss.getStageCounter() == lastStage) {
                    status = ss;
                    break;
                }
            }
            if (status == null)
                throw ServiceException.notFound();
            report = responseValidator.updateFormValidationReport(status, QuestionResponse.rollup(responsesByStage));
        }
        return report;
    }

    @EJB
    private StudyStatusNotifier studyStatusNotifier;
    @EJB // We use this to separate out the transactional contexts into appropriate lifetimes.
    private QuestionResponseUpdater questionResponseUpdater;
    
    @POST
    @Path("study/{id}/responses/{stage}")
    @TransactionAttribute(TransactionAttributeType.NEVER)
    public void setStudyResponses(@PathParam("id") String id, @PathParam("stage") Integer stage, Collection<QuestionResponse> responses, @QueryParam("user") Integer usersID) {
        studyStatusNotifier.notifyUpdate(questionResponseUpdater.setStudyResponses(id, stage, responses, usersID));
    }
        
    @POST
    @Path("study/{id}/responses/{stage}/feed")
    public QuestionResponseUpdates retrieveStudyResponseFeed(@PathParam("id") String id, @PathParam("stage") Integer stage, QuestionResponseUpdates request, @QueryParam("form") Integer formsID, @HeaderParam("sm_user") String loginName) throws InterruptedException {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", id).getSingleResult();
        try {
            User u = em.createNamedQuery("User.findByLoginName", User.class).setParameter("loginName", loginName).getSingleResult();
            Form f = em.find(Form.class, formsID);
            studyFormLock.renew(s, f, u);
        } catch (NoResultException ignoreBadLocking) {}
        for (StudyStatus ss : s.getStudyStatusCollection()) {
            if (ss.getStageCounter() == stage) {
                QuestionResponseUpdates response = new QuestionResponseUpdates();
                response.setResponses(new LinkedList<QuestionResponse>());
                if (!ss.getUpdatedAt().after(request.getUpdateTimestamp())) {
                    // We haven't gotten updates, so let's sleep on it.
                    if (studyStatusNotifier.awaitUpdate(ss, request.getUpdateTimestamp()))
                        ss = questionResponseUpdater.getUpdate(ss);
                }
                response.setUpdateTimestamp(ss.getUpdatedAt());
                for (QuestionResponse qr : ss.getQuestionResponseCollection()) {
                    if (qr.getUpdatedAt().after(request.getUpdateTimestamp()))
                        response.getResponses().add(qr);
                }
                response.setFormValidationReport(responseValidator.getFormValidationReport(ss)); // Must be in the if it changed since the call to GET .../rollup!
                return response;
            }
        }
        throw ServiceException.notFound();
    }
    
    @EJB
    private StudyFormLock studyFormLock;
    
    @PUT
    @Path("study/{id}/lock/{formid}")
    public void acquireLock(@PathParam ("id") String studyId,@PathParam("formid") Integer formId,@HeaderParam("sm_user") String loginName) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = em.createNamedQuery("User.findByLoginName",User.class).setParameter("loginName",loginName).getSingleResult();
        Form f = em.find(Form.class, formId);

        if (!studyFormLock.acquire(s,f,u))
            throw ServiceException.forbidden();
    }
    
    @DELETE
    @Path("study/{id}/lock/{formid}")
    public void releaseLock(@PathParam ("id") String studyId,@PathParam("formid") Integer formId,@HeaderParam("sm_user") String loginName) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = em.createNamedQuery("User.findByLoginName",User.class).setParameter("loginName",loginName).getSingleResult();
        Form f = em.find(Form.class, formId);
        
        if (!studyFormLock.release(s,f,u))
            throw ServiceException.forbidden();
    }

    @DELETE
    @Path("study/{id}/lock_a/{formid}")
    public void adminReleaseLock(@PathParam ("id") String studyId,@PathParam("formid") Integer formId,@HeaderParam("sm_user") String loginName) {
        Study s = em.createNamedQuery("Study.findByAssignedID", Study.class).setParameter("assignedID", studyId).getSingleResult();
        User u = em.createNamedQuery("User.findByLoginName",User.class).setParameter("loginName",loginName).getSingleResult();
        Form f = em.find(Form.class, formId);
        
        System.out.println(s);
        System.out.println(u);
        System.out.println(f);
        
        if (studyFormLock.administrativeRelease(s,f,u)){
            System.out.println("\nLock Released\n");
        }else{
            System.out.println("\nLock Not Released\n");
            throw ServiceException.forbidden();
        }
    }

    //temporary for debugging
    @GET
    @Produces(MediaType.TEXT_PLAIN)
    @Path("queryLock")
    public String queryLock() {
        return studyFormLock.inspect();
    }
    

}
