/*
* E1.0.0 RAM Increment 1 Final Code
* December 2014
* Department of Veterans Affairs/Booz Allen Hamilton 
* 
*/

package gov.va.rams.service;

import java.util.HashMap;
import java.util.Map;
import java.util.Calendar;

import gov.va.rams.entities.Form;
import gov.va.rams.entities.Study;
import gov.va.rams.entities.User;

import javax.ejb.Singleton;
import javax.ejb.LockType;
import javax.ejb.Lock;


@Singleton
public class StudyFormLock {
	private final Map<Study, Map<Form, UserCalendar>> locks = new HashMap<>();
	private static final int lock_timeout = (int)StudyStatusNotifier.MAX_WAIT_TIME * 2; // refresh rate is twice the timeout period
	
	@Lock(LockType.READ)
	public String inspect(){
		return locks.toString();
	}

	public boolean acquire(Study study, Form form, User user){
		Calendar expiration = Calendar.getInstance();
                expiration.add(Calendar.MILLISECOND, lock_timeout);

                Map<Form, UserCalendar> studyLocks = locks.get(study);
		//if map is completely empty or study is not keyed yet or form is not keyed yet or if time on the lock is older than 'now'
		if(studyLocks == null){
                        studyLocks = new HashMap<>();
			studyLocks.put(form,new UserCalendar(user,expiration));
			locks.put(study,studyLocks);
			return true;
		}

		if(studyLocks.get(form)==null
			|| studyLocks.get(form).getCalendar().before(Calendar.getInstance())){
			studyLocks.put(form,new UserCalendar(user,expiration));
			return true;
		}

		return false;	
	}

        
	public void renew(Study study, Form form, User user){
                Map<Form, UserCalendar> studyLocks = locks.get(study);
		if(studyLocks == null)
                        return;
                UserCalendar formLock = studyLocks.get(form);
		if(formLock == null || !formLock.getUser().equals(user))
                        return;

                Calendar expiration = Calendar.getInstance();
                expiration.add(Calendar.MILLISECOND, lock_timeout);
                formLock.setCalendar(expiration);
	}
	
	public boolean administrativeRelease(Study study, Form form, User user){
                Map<Form, UserCalendar> studyLocks = locks.get(study);
                if(studyLocks == null || studyLocks.remove(form) == null)
                        return false;
                if(studyLocks.isEmpty())
                        locks.remove(study);
                return true;
	}	
	
	public boolean release(Study study, Form form, User user){
		Map<Form, UserCalendar> studyLocks = locks.get(study);
		if(studyLocks == null)
                        return false;
		UserCalendar formLock = studyLocks.get(form);
                if(formLock == null || !formLock.getUser().equals(user))
                        return false;
                
		studyLocks.remove(form);
                if(studyLocks.isEmpty())
                        locks.remove(study);
		return true;
	}

}