/*
* E1.0.0 RAM Increment 1 Final Code
* December 2014
* Department of Veterans Affairs/Booz Allen Hamilton 
* 
*/

package gov.va.rams.service;

import gov.va.rams.entities.StudyStatus;
import java.util.Date;
import java.util.HashMap;
import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.Singleton;

/**
 * Wait up to {@link StudyStatusNotifier#MAX_WAIT_TIME} milliseconds
 * for a {@link StudyStatus} that the client is currently viewing to
 * be updated with new responses.
 */
@Singleton
@ConcurrencyManagement(ConcurrencyManagementType.BEAN)
public class StudyStatusNotifier {
    private final HashMap<StudyStatus, Date> latestUpdates = new HashMap<>();
    static final long MAX_WAIT_TIME = 5 * 60 * 1000;
    
    private synchronized Date getUpdateTimestamp(StudyStatus ss) {
        Date timestamp = latestUpdates.get(ss);
        if (timestamp == null) {
            timestamp = new Date(ss.getUpdatedAt().getTime());
            latestUpdates.put(ss, timestamp);
        }
        return timestamp;
    }
    
    public void notifyUpdate(StudyStatus ss) {
        final Date timestamp = getUpdateTimestamp(ss);
        synchronized (timestamp) {
            if (ss.getUpdatedAt().after(timestamp)) {
                timestamp.setTime(ss.getUpdatedAt().getTime());
                timestamp.notifyAll();
            }
        }
    }

    public boolean awaitUpdate(StudyStatus ss, Date since) throws InterruptedException {
        final Date timestamp = getUpdateTimestamp(ss);
        synchronized (timestamp) {
            timestamp.wait(MAX_WAIT_TIME);
            return timestamp.after(since);
        }
    }
}
