/***************************************************
E1.0.0 RAM Increment 1 Final Code
December 2014
Department of Veteran's Affairs/Booz Allen Hamilton
dynamicDashboard.js
***************************************************/

//Linking in the sql database
var restPath = "http://vaausramapp801.srv.domain:8080/RAMS-0.0.1/";
//Linking in the sharepoint site
var siteUrl = "http://ram-spdev.aac.dva.domain/sites/RAMS_Test/";

//template for username that is retrieved from sharepoint 
var retrievedUser = "testk";

//globals that have to be shared between functions
var facility, user, studyID, id;

//Initial function called by HTML. kicks off the process
//to grab the username from sharepoint
function process(){
    //loading connection to sharepoint
    var clientContext = new SP.ClientContext(siteUrl);

    this.website = clientContext.get_web();
    this.currentUser_SP = website.get_currentUser();
    clientContext.load(currentUser_SP);
    //trying to grab username information from sharepoint
    clientContext.executeQueryAsync(Function.createDelegate(this, this.userQuerySucceeded), Function.createDelegate(this, this.userQueryFailed));
};

//When we know that we have successfully retrieved a username from
//shareponit, then we can set up the baseline information that is needed.
function userQuerySucceeded(sender, args) {
    var username_SP, split;
    
    //parsing out the needed username from sharepoint
    username_SP = currentUser_SP.get_loginName();
    split = username_SP.split("\\");
    retrievedUser = split[1];

    //grabs a new generated study id
    var temp = sendREST("GET", "fascade/studyidgenerator/sitecode/1111/humorani/H", null, false);
    id = temp.studyAssignedID;

    //puts together the dashboard table
    createTable();
        
    //removes HTML traces that allow user to leave dashboard page
    $('.start').attr("onclick", "createStudy()");
    $('.start').attr("href", "irb-applications-general.html?" + id + "?" + retrievedUser);
	$('.start').html("Start IRB Application");
};

//notifying the user that their sharepoint username couldn't be retrieved
function userQueryFailed(sender, args) {
    alert('request failed ' + args.get_message() + '\n' + args.get_stackTrace());
};

//puts together the dashboard table. This will list out any studies that
//the user is associated with and provide information on those studies.
function createTable(){
    var recentStage, row, studies;
    var noStudies = true;
    
    //grabs all the studies that have been created.
    studies = sendREST("GET", "manage/study", null, false);

    //formatting the recieved JSON
    if(studies.study.length !== undefined){
        var studyLength = studies.study.length - 1;
        studyID = studies.study[studyLength].studiesID + 1; 
    }else{
	   studyID = 1;
    }

    //filtering through the studies and putting the necessary ones in the dashboard
    for(var i=0; i<studies.study.length; i++){
    	//accounting for recieved JSON
        if(studies.study[i].studyStatusCollection === undefined){
    	    continue;
    	}
    	
        //formatting the recieved JSON
    	if(studies.study[i].studyStatusCollection.length === undefined){
    	    studies.study[i].studyStatusCollection = [studies.study[i].studyStatusCollection];
        }

        //recentStage relates to whether or not the study is in draft or review
    	recentStage = studies.study[i].studyStatusCollection.length-1;
    		    
    	row = document.createElement("tr");
        if(studies.study[i].studyStatusCollection[recentStage].createdBy.displayName == retrievedUser){

    	    var cell = document.createElement("td");
    	    
            //allows the user to return to a previous study by clicking on this study ID link in the dashboard
    	    var anchor = document.createElement("a");
    	    anchor.setAttribute("href", "irb-applications-general.html?" + studies.study[i].assignedID + "?" + retrievedUser);
    	    anchor.setAttribute("title", studies.study[i].title);
    	    
            //study ID
    	    var text = document.createTextNode(studies.study[i].assignedID);
    	    anchor.appendChild(text);
    	    cell.appendChild(anchor);
    	    row.appendChild(cell);
    	    
    	    //study title
    	    row.appendChild(createCell(studies.study[i].title));
    	    
    	    //Type 
    	    row.appendChild(createCell("Type TBD"));
    	    
    	    //PI Name
    	    row.appendChild(createCell(studies.study[i].studyStatusCollection[recentStage].createdBy.displayName));
    	    
    	    //Status (draft or review)
    	    row.appendChild(createCell(studies.study[i].studyStatusCollection[recentStage].studyState.name));
    	    
    	    //Expiration Date
    	    row.appendChild(createCell("Expiration TBD"));
    	    
    	    //IRB Agenda Date
    	    row.appendChild(createCell("IRB DATE TBD"));
    	
    	    $('tbody').append(row);
    	    noStudies = false;
    	}	
    }

    if(noStudies){
    	//no studies linked to user
    	var cell = document.createElement("td")
        var text = document.createTextNode("You have no applications");
        cell.appendChild(text);
        cell.setAttribute("colspan", 7);
    	cell.setAttribute("style", "text-align:center");
    	row.appendChild(cell);	    
        
    	$('tbody').append(row);
    }
};

//creates a cell in a table. takes in what should be shown
//in the cell
function createCell(displayText){
    
    var cell = document.createElement("td");
    
    var text = document.createTextNode(displayText);
    cell.appendChild(text);
    
    return cell;
};

//when a user clicks "create a new IRB application", this enters the
//necessary information into the database to create a new study 
//and link it to the current user.
function createStudy(){
    var facilities, users, study;
    var userTable = new Hashtable();

    //a new study has to be linked to a facility
    facilities = sendREST("GET", "facility", null, false);
    if(facilities.facility.length === undefined)
        facilities.facility = [facilities.facility];
    
    //grabbing all the users in the database and formatting the recieved JSON
    users = sendREST("GET", "user", null, false);
    if(users.user.length === undefined)
        users.user = [users.user];
        
    if(facilities.facility.length == 0){
        //inserting a facility if the database doesn't hold any
        
        var addFacility = {
            "administratorSPGroup": 0,
            "description": "initial facility",
            "facilitiesID": 0,
            "name": "General Site",
            "reviewerSPCalendar": "00000000-0000-0000-0000-000000000000",
            "reviewerSPGroup": 0,
            "safetySPGroup": 0
        };
        
        facility = {
            "facility": addFacility
        };
        
        sendREST("POST", "manage/facility", facility, false);
    }

    //checking to see if the user is in the sql database
    for(var i=0; i<users.user.length; i++){
        userTable.put(users.user[i].displayName, users.user[i].usersID);
    }
    
    //adding the user to the database if need be
    if(!userTable.containsKey(retrievedUser)){
        var currentUser={
            "displayName": retrievedUser,
            "loginName": retrievedUser,
            "permanentLoginIdentifier": retrievedUser,
            "usersID": userTable.size()
        };
        
        user = {
            "user" : currentUser
        };
        
        userTable.put(currentUser.displayName, currentUser.usersID);
        sendREST("POST", "manage/user", user, false);
    }
    
    //packaging study
    facility = {
        "facilitiesID": facilities.facility[0].facilitiesID
    };

    study = {
        "assignedID": id,
        "facility": facility,
        "studiesID": studyID,
        "title": "New IRB Application"
    };

    var data = {
        "study": study
    };

    //putting the newly created study in the database
    sendREST("POST", "manage/study?user=" + userTable.get(retrievedUser), data, false);
};

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, sync){
    var xmlResponse;

    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
        xmlResponse = new XMLHttpRequest();
    }else{
        xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
        xmlResponse.open(httpMethod, restPath.concat(endpoint), sync);
        xmlResponse.setRequestHeader("Content-Type", "application/json");

        if((httpMethod == "PUT")||(httpMethod == "DELETE")){
            xmlResponse.setRequestHeader("sm_user", data);
            xmlResponse.send(null);

            //204 is the response status indicating success, while
            //403 indicates failure
            if(xmlResponse.status === 204){
                return true;
            }else if(xmlResponse.status === 403){
                return false;
            }
        }
        if(httpMethod == "POST"){
            try{
                //sending data
                xmlResponse.send(JSON.stringify(data));
            }catch(e){
                alert(e.toString());
            }
        }
        if(httpMethod == "GET"){
            //retrieving the needed data
            xmlResponse.onreadystatechange = function(){
                if(xmlResponse.readyState === 4){
                    if(xmlResponse.status === 200){
                        try{
                            xmlResponse = JSON.parse(xmlResponse.responseText);

                        }catch(e){
                            alert(e.toString());
                        }

                    }else{
                        alert(xmlResponse.statusText);
                    }
                }
            };
            xmlResponse.send(null);
            //passing the needed data to the function that is requesting it
            return xmlResponse;
        }

    }catch(e){
        alert(e.toString());
    }
};
